from .base import BaseMethod
from vk.types.responses import leads as m


class Leads(BaseMethod):
    async def check_user(
        self,
        lead_id: int = None,
        test_result: int = None,
        test_mode: bool = None,
        auto_start: bool = None,
        age: int = None,
        country: str = None,
    ):
        """
        Check if the user can start the lead.
        :param lead_id: Lead ID.
        :param test_result: Value to be return in 'result' field when test mode is used.
        :param test_mode:
        :param auto_start:
        :param age: User age.
        :param country: User country code.


        """
        method = self.get_method_name(self.check_user)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.CheckUser(**r)

    async def complete(
        self, vk_sid: str = None, secret: str = None, comment: str = None
    ):
        """
        Complete the lead started by user.
        :param vk_sid: Session obtained as GET parameter when session started.
        :param secret: Secret key from the lead testing interface.
        :param comment: Comment text.


        """
        method = self.get_method_name(self.complete)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Complete(**r)

    async def get_stats(
        self,
        lead_id: int = None,
        secret: str = None,
        date_start: str = None,
        date_end: str = None,
    ):
        """
        Return lead stats data.
        :param lead_id: Lead ID.
        :param secret: Secret key obtained from the lead testing interface.
        :param date_start: Day to start stats from (YYYY_MM_DD, e.g.2011-09-17).
        :param date_end: Day to finish stats (YYYY_MM_DD, e.g.2011-09-17).


        """
        method = self.get_method_name(self.get_stats)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetStats(**r)

    async def get_users(
        self,
        offer_id: int = None,
        secret: str = None,
        offset: int = None,
        count: int = None,
        status: int = None,
        reverse: bool = None,
    ):
        """
        Return a list of last user actions for the offer.
        :param offer_id: Offer ID.
        :param secret: Secret key obtained in the lead testing interface.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.
        :param status: Action type. Possible values: *'0' — start,, *'1' — finish,, *'2' — blocking users,, *'3' — start in a test mode,, *'4' — finish in a test mode.
        :param reverse: Sort order. Possible values: *'1' — chronological,, *'0' — reverse chronological.


        """
        method = self.get_method_name(self.get_users)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetUsers(**r)

    async def metric_hit(self, data: str = None):
        """
        Count the metric event.
        :param data: Metric data obtained in the lead interface.


        """
        method = self.get_method_name(self.metric_hit)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MetricHit(**r)

    async def start(
        self,
        lead_id: int = None,
        secret: str = None,
        uid: int = None,
        aid: int = None,
        test_mode: bool = None,
        force: bool = None,
    ):
        """
        Create new session for the user passing the offer.
        :param lead_id: Lead ID.
        :param secret: Secret key from the lead testing interface.
        :param uid:
        :param aid:
        :param test_mode:
        :param force:


        """
        method = self.get_method_name(self.start)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Start(**r)



"""
A simple abstraction to build the largest bots and sharing rules between handlers.

Example usage:

    You have a simple bot with admin and user commands[handlers],
    what to limit access to admin commands[handlers] you create a simple rule[named rule, rule] for passing him
    in message_handler decorator. If you have a really big count of commands, you need write this:

    .. code-block:: python3
        @dp.message_handler(is_admin=True)
        async def some(msg, data):
            ...

    repeatedly. We appreciate your time and we create blueprints for this simple cases.

    But! If you have the largest bot in the world with lots of handlers, you need a simple, and more powerful
    tool to register handlers.
    Blueprint registering looks like that:

    .. code-block:: python3
        blueprint = Blueprint(...)
        dp.setup_blueprint(blueprint)

    It's easy!

"""
import typing
from abc import ABC
from abc import abstractmethod

from vk.bot_framework.dispatcher.rule import BaseRule
from vk.types import BotEvent as Event
from vk.utils.mixins import MetaMixin


class HandlerInBlueprint(typing.NamedTuple, MetaMixin):
    coro: typing.Callable
    event_type: Event
    rules: typing.List[BaseRule]
    named_rules: typing.List[BaseRule]
    meta: dict = {}


class AbstractBlueprint(ABC, MetaMixin):
    @abstractmethod
    def message_handler(
        self,
        *rules: typing.Tuple[typing.Type[BaseRule]],
        **named_rules: typing.Dict[str, typing.Any],
    ):
        """
        Register a message handler by the blueprint.
        :param rules:
        :param named_rules:
        :return:
        """

    @abstractmethod
    def event_handler(
        self,
        event_type: Event,
        *rules: typing.Tuple[typing.Type[BaseRule]],
        **named_rules: typing.Dict[str, typing.Any],
    ):
        """
        Register an event handler by the blueprint.
        :param event_type:
        :param rules:
        :param named_rules:
        :return:
        """


class Blueprint(AbstractBlueprint):
    def __init__(self, *rules: typing.Tuple[BaseRule], **named_rules):
        self.default_rules = rules
        self.default_named_rules = named_rules

        self._handlers: typing.List[HandlerInBlueprint] = []

        self._name = "A yet another blueprint"
        self._description = "Hm..."

    @property
    def handlers(self):
        return self._handlers

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @name.setter
    def name(self, new_name: str):
        self._name = new_name

    @description.setter
    def description(self, new_description: str):
        self._description = new_description

    def get_handler(self, handler_coro: typing.Callable):
        """
        Get handler object by the handler coroutine.
        :param handler_coro:
        :return:
        """
        for handler in self.handlers:
            if handler.coro is handler_coro:
                return handler

    def described_handler(
        self,
        name: str = None,
        description: str = None,
        deprecated: bool = False,
        **other_meta: dict,
    ):
        def decorator(coro: typing.Callable):
            handler = self.get_handler(coro)
            if not handler:
                raise RuntimeError("Handler not registered.")
            meta = {
                "name": name,
                "description": description,
                "deprecated": deprecated,
                **other_meta,
            }
            if handler.coro.__doc__:  # or match description in docstring
                meta["description"] = handler.coro.__doc__.strip()
            handler.meta.update(
                {k: v for k, v in meta.items() if v is not None}
            )

        return decorator

    def message_handler(
        self,
        *rules: typing.Tuple[
            typing.Type[BaseRule], typing.Callable, typing.Awaitable
        ],
        **named_rules: typing.Dict[str, typing.Any],
    ):
        """
        Register message handler by the decorator.

        :param rules: other user rules
        :param named_rules: other user named rules
        :return:
        """

        def decorator(coro: typing.Callable):
            nonlocal rules, named_rules

            rules = list(rules)
            rules.extend(self.default_rules)
            named_rules.update(self.default_named_rules)

            self.handlers.append(
                HandlerInBlueprint(
                    coro, Event.MESSAGE_NEW, rules, named_rules, {}
                )
            )
            return coro

        return decorator

    def event_handler(
        self,
        event_type: Event,
        *rules: typing.Tuple[typing.Type[BaseRule]],
        **named_rules: typing.Dict[str, typing.Any],
    ):
        def decorator(coro: typing.Callable):
            nonlocal rules, named_rules

            rules = list(rules)
            rules.extend(self.default_rules)
            named_rules.update(self.default_named_rules)

            self.handlers.append(
                HandlerInBlueprint(coro, event_type, rules, named_rules, {})
            )
            return coro

        return decorator



from .base import BaseModel

from vk.types.additional import City, Country
from vk.types.attachments import CropPhoto
from enum import IntEnum

import typing

# https://vk.com/dev/objects/user


class UserCareer(BaseModel):
    group_id: int = None
    company: str = None
    country_id: int = None
    city_id: int = None
    city_name: str = None
    from_: int = None
    until: int = None
    position: str = None


class UserCounters(BaseModel):
    albums: int = None
    videos: int = None
    audios: int = None
    photo: int = None
    notes: int = None
    friends: int = None
    groups: int = None
    online_friends: int = None
    mutual_friends: int = None
    user_videos: int = None
    followers: int = None
    pages: int = None


class UserContacts(BaseModel):
    mobile_phone: str = None
    home_phone: str = None


class UserEducation(BaseModel):
    university: int = None
    university_name: str = None
    faculty: int = None
    faculty_name: str = None
    graduation: int = None


class UserPlatform(IntEnum):
    m_vk_com = 1
    iphone = 2
    ipad = 3
    android = 4
    windows_phone_app = 5
    windows_8_app = 6
    web = 7
    vk_com = 7
    vk_mobile = 8


class UserLastSeen(BaseModel):
    time: int = None
    platform: UserPlatform = None


class UserMilitary(BaseModel):
    unit: str = None
    unit_id: int = None
    country_id: int = None
    from_: int = None
    until: int = None


class UserOccupation(BaseModel):
    type: str = None
    id: int = None
    name: str = None


class Political(IntEnum):
    communist = 1
    socialist = 2
    moderate = 3
    liberal = 4
    conservative = 5
    monarchist = 6
    ultraconservative = 7
    apathetic = 8
    libertarian = 9


class PeopleMain(IntEnum):
    intelect = 1
    creativity = 1
    kindness = 2
    honesty = 2
    health = 3
    beauty = 3
    wealth = 4
    power = 4
    courage = 5
    persistance = 5
    humor = 6
    love_for_life = 6


class LifeMain(IntEnum):
    family = 1
    children = 1
    career = 2
    money = 2
    entertainment = 3
    leisure = 3
    science = 4
    research = 4
    improving_the_world = 5
    personal_development = 6
    beaty = 7
    art = 7
    fame = 8
    influence = 8


class Smoking(IntEnum):
    very_negative = 1
    negative = 2
    neutral = 3
    compromisable = 4
    positive = 5


class Alcohol(BaseModel):
    very_negative = 1
    negative = 2
    neutral = 3
    compromisable = 4
    positive = 5


class UserPersonal(BaseModel):
    political: Political = None
    langs: typing.List[str] = None
    religion: str = None
    inspired_by: str = None
    people_main: PeopleMain = None
    life_main: LifeMain = None
    smoking: Smoking = None
    alcohol: Alcohol = None


class UserRelation(IntEnum):
    single = 1
    in_relationship = 2
    engaged = 3
    married = 4
    complicated = 5
    actively_searching = 6
    in_love = 7
    in_civil_union = 8
    not_specified = 0


class SchoolType(IntEnum):
    school = 0
    gymnasium = 1
    lyceum = 2
    boarding_school = 3
    evening_school = 4
    music_school = 5
    sport_school = 6
    artistic_school = 7
    college = 8
    professional_lyceum = 9
    technical_college = 10
    vocational = 11
    specialized_school = 12
    art_school = 13


class School(BaseModel):
    id: int = None
    country: int = None
    city: int = None
    name: str = None
    year_from: int = None
    year_to: int = None
    year_graduated: int = None
    class_: str = None
    speciality: str = None
    type: int = None
    type_str: SchoolType = None


class Universitiy(BaseModel):
    id: int = None
    country: int = None
    city: int = None
    name: str = None
    faculty: int = None
    faculty_name: str = None
    chair: int = None
    chair_name: str = None
    graduation: int = None
    education_form: str = None
    education_status: str = None


class User(BaseModel):
    id: int = None
    first_name: str = None
    last_name: str = None
    deactivated: str = None
    is_closed: bool = None
    can_access_closed: bool = None
    about: str = None
    activities: str = None
    bdate: str = None
    blacklisted: int = None
    blacklisted_by_me: int = None
    books: str = None
    can_post: int = None
    can_see_all_posts: int = None
    can_see_audio: int = None
    can_send_friend_request: int = None
    can_write_private_message: int = None
    career: UserCareer = None
    city: City = None
    common_count: int = None
    contacts: UserContacts = None
    counters: UserCounters = None
    country: Country = None
    crop_photo: CropPhoto = None
    domain: str = None
    education: UserEducation = None
    followers_count: int = None
    friend_status: int = None
    games: str = None
    has_mobile: int = None
    has_photo: int = None
    home_town: str = None
    interests: str = None
    is_favorite: int = None
    is_friend: int = None
    is_hidden_from_feed: int = None
    last_seen: UserLastSeen = None
    lists: typing.List[int] = None
    maiden_name: str = None
    military: UserMilitary = None
    movies: str = None
    music: str = None
    nickname: str = None
    occupation: UserOccupation = None
    online: int = None
    personal: UserPersonal = None
    photo_50: str = None
    photo_100: str = None
    photo_200_orig: str = None
    photo_200: str = None
    photo_400_orig: str = None
    photo_id: str = None
    photo_max: str = None
    photo_max_orig: str = None
    quotes: str = None
    relatives: typing.List = None
    relation: UserRelation = None
    schools: typing.List[School] = None
    screen_name: str = None
    sex: int = None
    site: str = None
    status: str = None
    timezone: int = None
    trending: int = None
    tv: str = None
    universities: typing.List[Universitiy] = None
    verified: int = None
    wall_comments: int = None



from vk import VK
from vk.exceptions import APIException
from vk.types import Message
from vk.types import User
from vk.types.community import Community
from vk.utils import ContextInstanceMixin


class Validator(ContextInstanceMixin):
    def __init__(self, vk: VK = None):
        self._vk = vk if vk else VK.get_current()
        self._api = vk.get_api()
        self.validators_answers: dict = {
            "valid_vk_id": "",
            "positive_number": "",
            "valid_vk_group_id": "",
        }

    @property
    def vk(self):
        return self._vk

    @property
    def api(self):
        return self._api

    async def valid_vk_group_id(self, arg: str, message: Message):
        """
        Validate passed in message group_id or screenname.
        Similar to 'valid_vk_id'.

        If everything is good, append a response from VKAPI to the field 'valid_vk_group_id_group'.
        :param arg:
        :param message:
        :return:
        """
        have_answer = self.validators_answers["valid_vk_group_id"]
        try:
            result = await self.vk.api_request(
                "groups.getById", {"group_id": arg}
            )
        except APIException:
            if have_answer:
                await message.answer(have_answer)
            return False
        if not result and have_answer:
            await message.answer(have_answer)
        if result:
            return {"valid_vk_group_id_group": Community(**result[0])}
        return result

    async def valid_vk_id(self, arg: str, message: Message):
        """
        Validate passed in message ID or screenname.
        Search user that obtain that ID (screenname).

        If everything is good, append a response from VKAPI to the field 'valid_vk_id_user'.
        Instance:

        @dp.message_handler(commands=["hello"], have_args=[validators.valid_vk_id])
        async def handle(message: types.Message, data: dict):
            usr: types.User = data["valid_vk_id_user"]
            await message.answer(usr.first_name)

        :param message:
        :param arg:
        :return:
        """

        have_answer = self.validators_answers["valid_vk_id"]
        try:
            result = await self.vk.api_request("users.get", {"user_ids": arg})
        except APIException:
            if have_answer:
                await message.answer(have_answer)
            return False
        if not result and have_answer:
            await message.answer(have_answer)
        if result:
            return {"valid_vk_id_user": User(**result[0])}
        return result

    async def positive_number(self, arg: str, message: Message):
        """
        :param message:
        :param arg:
        :return: str is number and it is positive
        """
        have_answer = self.validators_answers["positive_number"]
        if not arg.isdigit():
            if have_answer:
                await message.answer(have_answer)
            return False
        if int(arg) <= 0:
            if have_answer:
                await message.answer(have_answer)
            return False
        return True



from . import others

from . import account
from . import apps
from . import appwidgets
from . import auth
from . import board
from . import database
from . import docs
from . import fave
from . import friends
from . import gift
from . import groups
from . import leadforms
from . import leads
from . import likes
from . import market
from . import messages



"""
Author: https://github.com/aiogram/aiogram/blob/dev-2.x/aiogram/utils/mixins.py
"""
import contextvars
from enum import Enum
from typing import Type
from typing import TypeVar
from typing import Union

from pydantic import BaseModel
from pydantic import validator

T = TypeVar("T")


class ContextInstanceMixin:
    def __init_subclass__(cls, **kwargs):
        cls.__context_instance = contextvars.ContextVar(
            "instance_" + cls.__name__
        )
        return cls

    @classmethod
    def get_current(cls: Type[T], no_error=True) -> T:
        if no_error:
            return cls.__context_instance.get(None)
        return cls.__context_instance.get()

    @classmethod
    def set_current(cls: Type[T], value: T):
        if not isinstance(value, cls):
            raise TypeError(
                f"Value should be instance of '{cls.__name__}' not '{type(value).__name__}'"
            )
        cls.__context_instance.set(value)


class _MetaObjectType(Enum):
    HANDLER = "handler"
    MIDDLEWARE = "middleware"
    BLUEPRINT = "blueprint"
    EXTENSION = "extension"
    RULE = "rule"

    UNKNOWN = "unknown"


class _MetaModel(BaseModel):
    name: str = str
    description: str = str
    deprecated: bool = False
    type: _MetaObjectType = _MetaObjectType.UNKNOWN

    class Config:
        extra = "allow"

    @validator("type", pre=True)
    def validate_type(cls, v: Union[_MetaObjectType, str]):
        if isinstance(v, _MetaObjectType):
            return v
        elif isinstance(v, str):
            try:
                as_enum_value = _MetaObjectType(v)
            except ValueError:
                as_enum_value = _MetaObjectType.UNKNOWN
            return as_enum_value
        else:
            raise TypeError("Unallowed type for field")

    def get(self, key, default=None):
        """
        Be like `dict`.
        :param key:
        :param default:
        :return:
        """
        return getattr(self, key, default)

    def __str__(self):
        args = ", ".join(
            [f"{key}={value}" for key, value in self.dict().items()]
        )
        return "{}({})".format(self.__class__.__name__, args)


class MetaMixin:

    # default meta tags:
    # name: str
    # description: str
    # deprecated: bool
    # type: _MetaObjectType or str

    # meta = {
    # "name": "My object which contain meta variable",
    # "description": "Oh... i don't know..",
    # "deprecated": False,
    # }
    # it will be converted to `_MetaModel` with `dot-access` to attributes.

    meta: Union[
        dict, _MetaModel
    ] = {}  # information about object special for third-party-addons.

    def __init_subclass__(cls, **kwargs):
        if isinstance(cls.meta, dict):
            new_meta = _MetaModel(**cls.meta)
        elif isinstance(cls.meta, _MetaModel):
            new_meta = cls.meta
        else:
            raise TypeError("Unallowed type for meta property")
        cls.meta = new_meta



from .others import SimpleResponse
from ..base import BaseModel

from ..appwidget import AppWidgetImage

import typing


class GetAppImageUploadServerResponse(BaseModel):
    upload_url: str = None


class GetAppImageUploadServer(BaseModel):
    response: GetAppImageUploadServerResponse = None


class GetAppImagesResponse(BaseModel):
    count: int = None
    items: typing.List[AppWidgetImage] = None


class GetAppImages(BaseModel):
    response: GetAppImagesResponse = None


class GetGroupImageUploadServer(GetAppImageUploadServer):
    pass


class GetGroupImages(GetAppImages):
    pass


class GetImagesById(BaseModel):
    response: AppWidgetImage = None


class SaveAppImage(GetImagesById):
    pass


class SaveGroupImage(SaveAppImage):
    pass


class Update(SimpleResponse):
    pass



class APIException(Exception):
    def __init__(self, code: int, text: str):
        self.code = code
        self.text = text
        self.message = f"[{self.code}] {self.text}"
        super().__init__(self.message)


class KeyboardException(Exception):
    pass



import typing

from ..additional import Email
from ..attachments import Photo
from ..base import BaseModel
from ..chat import Chat
from ..community import Community
from ..conversation import Conversation
from ..message import Message
from ..user import User
from .others import SimpleResponse


class AddChatUser(SimpleResponse):
    pass


class AllowMessagesFromGroup(SimpleResponse):
    pass


class CreateChat(SimpleResponse):
    pass


class Delete(BaseModel):
    response: typing.Union[dict, int] = None


class DeleteChatPhotoResponse(BaseModel):
    message_id: int = None
    chat: Chat = None


class DeleteChatPhoto(BaseModel):
    response: DeleteChatPhotoResponse = None


class DeleteConversation(SimpleResponse):
    pass


class DenyMessagesFromGroup(SimpleResponse):
    pass


class Edit(SimpleResponse):
    pass


class EditChat(SimpleResponse):
    pass


class GetByConversationMessageIdResponse(BaseModel):
    count: int = None
    items: typing.List[Message] = None


class GetByConversationMessageId(BaseModel):
    response: GetByConversationMessageIdResponse = None


class GetById(BaseModel):
    response: typing.Union[
        GetByConversationMessageIdResponse,
        typing.List[GetByConversationMessageId],
    ] = None


class GetChat(BaseModel):
    response: Chat = None


class ChatMembers(BaseModel):
    profiles: typing.List[User] = None
    groups: typing.List[Community] = None
    email: typing.List[Email] = None


class GetChatPreviewResponse(BaseModel):
    admin_id: int = None
    members: ChatMembers = None
    title: str = None
    photo: Photo


class GetChatPreview(BaseModel):
    response: GetChatPreviewResponse = None


class GetConversationMembersResponseItem(BaseModel):
    member_id: int = None
    invited_by: int = None
    join_date: int = None
    is_admin: bool = None
    is_owner: bool = None


class GetConversationMembersResponse(BaseModel):
    count: int = None
    items: typing.List[GetConversationMembersResponseItem] = None
    profiles: typing.List[User] = None
    groups: typing.List[Community] = None


class GetConversationMembers(BaseModel):
    response: GetConversationMembersResponse = None


class GetConversationsItem(BaseModel):
    conversation: Conversation = None
    last_message: Message = None


class GetConversationsResponse(BaseModel):
    count: int = None
    items: typing.List[GetConversationsItem] = None
    unread_count: int = None
    profiles: typing.List[User] = None
    groups: typing.List[Community] = None


class GetConversations(BaseModel):
    response: GetConversationsResponse = None


class GetConversationsByIdResponse(BaseModel):
    count: int = None
    items: typing.List[Conversation] = None
    profiles: typing.List[User] = None


class GetConversationsById(BaseModel):
    response: GetConversationsByIdResponse = None


class GetHistoryResponse(BaseModel):
    count: int = None
    items: typing.List[Message] = None


class GetHistory(BaseModel):
    response: GetHistoryResponse = None


class GetHistoryAttachments(BaseModel):
    response: typing.Any = None


class GetImportantMessagesResponseMessages(BaseModel):
    count: int = None
    items: typing.List[Message] = None


class GetImportantMessagesResponse(BaseModel):
    messages: GetImportantMessagesResponseMessages = None


class GetImportantMessages(BaseModel):
    response: GetImportantMessagesResponse = None


class GetInviteLinkResponse(BaseModel):
    link: str = None


class GetInviteLink(BaseModel):
    response: GetInviteLinkResponse = None


class GetLastActivityResponse(BaseModel):
    online: int = None
    time: int = None


class GetLastActivity(BaseModel):
    response: GetLastActivityResponse = None


class GetLongPollHistoryResponse(BaseModel):
    history: typing.List[typing.List[int]] = None
    messages: GetImportantMessagesResponseMessages = None
    profiles: typing.List[User] = None
    new_pts: int = None


class GetLongPollHistory(BaseModel):
    response: GetLongPollHistoryResponse = None


class GetLongPollServerResponse(BaseModel):
    key: str = None
    server: str = None
    ts: int = None


class GetLongPollServer(BaseModel):
    response: GetLongPollServerResponse = None


class IsMessagesFromGroupAllowedResponse(BaseModel):
    is_allowed: int = None


class IsMessagesFromGroupAllowed(BaseModel):
    response: IsMessagesFromGroupAllowedResponse = None


class JoinChatByInviteLinkResponse(BaseModel):
    chat_id: int = None


class JoinChatByInviteLink(BaseModel):
    response: JoinChatByInviteLinkResponse = None


class MarkAsAnsweredConversation(SimpleResponse):
    pass


class MarkAsImportant(BaseModel):
    response: typing.List[int] = None


class MarkAsImportantConversation(SimpleResponse):
    pass


class MarkAsRead(SimpleResponse):
    pass


class Pin(SimpleResponse):
    pass


class RemoveChatUser(SimpleResponse):
    pass


class Restore(SimpleResponse):
    pass


class SearchResponse(BaseModel):
    count: int = None
    items: typing.List[Message] = None


class Search(BaseModel):
    response: SearchResponse = None


class SeacrhConversationsResponse(BaseModel):
    count: int = None
    items: typing.List[Conversation] = None


class SearchConversations(BaseModel):
    response: SeacrhConversationsResponse = None


class Send(SimpleResponse):
    pass


class SetActivity(SimpleResponse):
    pass


class SetChatPhotoResponse(BaseModel):
    message_id: int = None
    chat: Chat = None


class SetChatPhoto(BaseModel):
    response: SetChatPhotoResponse = None


class Unpin(SimpleResponse):
    pass



import logging

from ..dispatcher.handler import SkipHandler
from ..dispatcher.middleware import BaseMiddleware
from vk import types

logger = logging.getLogger(__name__)

"""
Built-in middlewares.
"""


class OnlyMessagesMiddleware(BaseMiddleware):
    meta = {
        "name": "OnlyMessagesMiddleware",
        "description": "A simple middleware for skipping not 'message_new' events.",
        "deprecated": False,
    }

    async def pre_process_event(
        self, event: types.MessageNew, data: dict
    ) -> dict:
        if event.type == "message_new":
            logger.info("New message! Handlers will not be skipped.")
            return data
        else:
            logger.info("Not message. Handlers will be skipped.")
            raise SkipHandler()



import typing

from .additional import PostSource
from .attachments import Attachment
from .attachments import Geo
from .attachments import Like
from .attachments import Repost
from .base import BaseModel

# https://vk.com/dev/objects/post


class WallPostComments(BaseModel):
    count: int = None
    can_post: int = None
    groups_can_post: int = None


class WallPost(BaseModel):
    id: int = None
    owner_id: int = None
    from_id: int = None
    created_by: int = None
    date: int = None
    text: str = None
    reply_owner_id: int = None
    reply_post_id: int = None
    friends_only: int = None
    comments: WallPostComments = None
    likes: Like = None
    reposts: Repost = None
    post_type: str = None
    post_source: PostSource = None
    attachments: typing.List[Attachment] = None
    geo: Geo = None
    signer_id: int = None
    copy_history: typing.Any = None
    can_pin: int = None
    can_delete: int = None
    can_edit: int = None
    is_pinned: int = None
    marked_as_ads: int = None
    is_favorite: bool = None



from ..base import BaseModel

# returned from https://vk.com/dev/account.getActiveOffers


class ActiveOffer(BaseModel):
    id: str = None
    title: str = None
    instruction: str = None
    instruction_html: str = None
    short_description: str = None
    description: str = None
    img: str = None
    tag: str = None
    price: int = None



import logging
from abc import ABC
from abc import abstractmethod

from vk.utils.mixins import MetaMixin

logger = logging.getLogger(__name__)


class AbstractRule(ABC, MetaMixin):
    @abstractmethod
    async def check(self, event, data: dict):
        """
        This method is gonna be called in the rules check.

        :param data:
        :param event:
        :return: True or False. True -> check next rules or execute handler
        """


class BaseRule(AbstractRule, ABC):
    async def __call__(self, event, data: dict) -> bool:
        logger.debug(f"Rule {self.__class__.__name__} succesfully called!")
        return await self.check(event, data)


class NamedRule(BaseRule, ABC):
    """
    Can be added to the list of rules with RuleFactory and
    is going to be called in handlers by unique key;

    >>> @dp.message_handler(unique_key = value)

    """

    key = None  # unique value for access to rule in handlers.
    required = False  # include to all handlers rules.
    default = (
        None
    )  # default value for the rule


class RuleFactory:
    """
    Manage your rules.
    """

    def __init__(self, config: dict):
        self.config = config  # dict of all known rules

    def setup(self, rule: NamedRule):
        """
        Register rule in factory.
        :param rule:
        :return:
        """
        if not issubclass(rule, NamedRule):
            raise RuntimeError("Only NamedRules may be added in rule factory!")

        if rule.key is None or not isinstance(rule.key, str):
            raise RuntimeError("Unallowed key for rule")

        self.config.update({rule.key: rule})
        logger.debug(f"Rule {rule.__class__.__name__} succesfully added!")

    def get_rules(self, user_rules: dict):
        """
        Get rules objects by named_rules.
        :param user_rules:
        :return:
        """
        rules = []
        for key, value in user_rules.items():
            if key in self.config:
                rule: BaseRule = self.config[key](value)
                if rule.meta and rule.meta.get("deprecated", False):
                    logger.warning(
                        f"This rule ({rule.__class__.__name__}) deprecated. Not recommended to use."
                    )
                rules.append(rule)
                continue
            else:
                raise RuntimeError(f"Unknown rule passed: {key}")
        # include required rules
        k: str
        v: NamedRule
        for k, v in self.config.items():
            if v.required:
                this_rule_not_included = True
                for rule in rules:
                    if isinstance(rule, v):
                        this_rule_not_included = False
                        break
                if this_rule_not_included:
                    rules.append(v(v.default))
        return rules



import vk.utils.vkscript.handlers.assignments
import vk.utils.vkscript.handlers.blocks
import vk.utils.vkscript.handlers.calls
import vk.utils.vkscript.handlers.expressions
import vk.utils.vkscript.handlers.statements
import vk.utils.vkscript.handlers.types
from .converter import VKScriptConverter
from .execute import Execute
from .execute import execute


__all__ = ("execute", "Execute", "VKScriptConverter")



from .base import BaseMethod
from vk.types.responses import auth as m


class Auth(BaseMethod):
    async def check_phone(
        self,
        phone: str = None,
        client_id: int = None,
        client_secret: str = None,
        auth_by_phone: bool = None,
    ):
        """
        Check a user's phone number for correctness.
        :param phone: Phone number.
        :param client_id: User ID.
        :param client_secret:
        :param auth_by_phone:


        """
        method = self.get_method_name(self.check_phone)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.CheckPhone(**r)

    async def restore(self, phone: str = None, last_name: str = None):
        """
        Allow to restore account access using a code received via SMS. " This method is only available for apps with [vk.com/dev/auth_direct|Direct authorization] access. "
        :param phone: User phone number.
        :param last_name: User last name.


        """
        method = self.get_method_name(self.restore)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Restore(**r)



from ..base import BaseModel
from .like import Like
from .attachments import Attachments

# https://vk.com/dev/objects/topic
# https://vk.com/dev/objects/comment_board


class TopicComment(BaseModel):
    id: int = None
    from_id: int = None
    date: int = None
    text: str = None
    attachments: Attachments = None
    likes: Like = None


class Topic(BaseModel):
    id: int = None
    title: str = None
    created: int = None
    created_by: int = None
    updated: int = None
    updated_by: int = None
    is_closed: int = None
    is_fixed: int = None
    comments: int = None
    first_comment: str = None
    last_comment: str = None



import typing

from ..vk import VK
from ..constants import JSON_LIBRARY

from vk.types.attachments import Attachment
from vk.types.attachments.geo import Geo
from ..types.base import BaseModel


class StreamingRule(BaseModel):
    value: str
    tag: str


class StreamingError(BaseModel):
    message: str
    error_code: int


class StreamingResponse(BaseModel):
    code: int


class StreamingGetResponse(StreamingResponse):
    rules: typing.List[StreamingRule] = None
    error: StreamingError = None


class StreamingAddResponse(StreamingResponse):
    error: StreamingError = None


class StreamingServiceMessage(BaseModel):
    message: str
    service_code: int


class StreamingEventId(BaseModel):
    post_owner_id: int = None
    post_id: int = None
    comment_id: int = None
    shared_post_id: int = None
    topic_owner_id: int = None
    topic_id: int = None
    topic_post_id: int = None


class Author(BaseModel):
    id: int
    author_url: str
    shared_post_author_id: int = None
    shared_post_author_url: str = None
    platform: int


class StreamingEvent(BaseModel):
    event_type: str = None
    event_id: StreamingEventId = None
    event_url: str = None
    text: str = None
    action: str = None
    action_time: int = None
    creation_time: int = None
    attachments: typing.List[Attachment] = []
    geo: Geo = None
    shared_post_text: str = None
    shared_post_creation_time: int
    signer_id: int = None
    tags: typing.List[str] = []
    author: Author = None


class StreamingReadResponse(StreamingResponse):
    service_message: StreamingServiceMessage = None
    event: StreamingEvent = None


class Streaming:
    def __init__(self, vk: VK):
        self.vk: VK = vk
        self._server = None
        self._server_ws = None

    async def get_server(self):
        resp = await self.vk.api_request("streaming.getServerUrl")
        endpoint = resp["endpoint"]
        key = resp["key"]
        self._server = f"https://{endpoint}/rules?key={key}"
        self._server_ws = f"https://{endpoint}/stream?key={key}"

    async def get_rules(self):
        async with self.vk.client.get(self._server) as resp:
            json = await resp.json(loads=JSON_LIBRARY.loads)
            return StreamingGetResponse(**json)

    async def add_rule(self, value: str, tag: str):
        rule = {"rule": {"value": value, "tag": tag}}
        async with self.vk.client.post(self._server, params=rule) as resp:
            json = await resp.json(loads=JSON_LIBRARY.loads)
            return StreamingAddResponse(**json)

    async def delete_rule(self, tag: str):
        tag = {"tag": tag}
        async with self.vk.client.put(self._server, params=tag) as resp:
            json = await resp.json(loads=JSON_LIBRARY.loads)
            return StreamingAddResponse(**json)

    async def read(self):
        async with self.vk.client.ws_connect(self._server_ws) as ws:
            async for msg in ws:
                data = JSON_LIBRARY.loads(msg)
                yield StreamingReadResponse(**data)



from .others import SimpleResponse
from ..base import BaseModel

import typing


class AddArticle(SimpleResponse):
    pass


class AddLink(SimpleResponse):
    pass


class AddPage(SimpleResponse):
    pass


class AddPost(SimpleResponse):
    pass


class AddProduct(SimpleResponse):
    pass


class AddTagResponse(BaseModel):
    id: int = None
    name: str = None


class AddTag(BaseModel):
    response: AddTagResponse = None


class AddVideo(SimpleResponse):
    pass


class EditTag(SimpleResponse):
    pass


class Get(BaseModel):
    response: typing.Any = None


class GetPages(BaseModel):
    response: typing.Any = None


class GetTagsResponse(BaseModel):
    count: int = None
    items: typing.List[AddTagResponse] = None


class GetTags(BaseModel):
    response: GetTagsResponse = None


class MarkSeen(SimpleResponse):
    pass


class RemoveArticle(SimpleResponse):
    pass


class RemoveLink(SimpleResponse):
    pass


class RemovePage(SimpleResponse):
    pass


class RemovePost(SimpleResponse):
    pass


class RemoveProduct(SimpleResponse):
    pass


class RemoveTag(SimpleResponse):
    pass


class RemoveVideo(SimpleResponse):
    pass


class ReorderTags(SimpleResponse):
    pass


class SetPageTags(SimpleResponse):
    pass


class SetTags(SimpleResponse):
    pass


class TrackPageInteraction(SimpleResponse):
    pass



from enum import IntEnum


class AudioGenres(IntEnum):
    rock = 1
    pop = 2
    rap = 3
    hip_hop = 3
    easy_listening = 4
    house = 5
    dance = 5
    instrumental = 6
    metal = 7
    alternative = 21
    dubstep = 8
    jazz = 1001
    blues = 1001
    drum = 10
    bass = 10
    trance = 11
    chanson = 12
    ethnic = 13
    acoustic = 14
    vocal = 14
    reggat = 15
    classical = 16
    indie_pop = 17
    other = 18
    speech = 19
    electropop = 22
    disco = 22



import asyncio
import logging
import typing

from .auto_reload import _auto_reload

logger = logging.getLogger(__name__)


class TaskManager:
    """
    Task manager represent to user high-level API of asyncio interface (Less part :))
    """

    def __init__(self, loop: asyncio.AbstractEventLoop):
        self.tasks: typing.List[typing.Callable] = []
        self.loop: asyncio.AbstractEventLoop = loop

    def run(
        self,
        *,
        on_shutdown: typing.Callable = None,
        on_startup: typing.Callable = None,
        asyncio_debug_mode: bool = False,
        auto_reload: bool = False,
    ):
        """
        Method which run event loop

        :param auto_reload: auto reload code when changes
        :param on_shutdown: coroutine which runned after complete tasks
        :param on_startup: coroutine which runned before start main tasks
        :param asyncio_debug_mode: asyncio debug mode state
        :return:
        """
        if len(self.tasks) < 1:
            raise RuntimeError("Count of tasks - 0. Add tasks.")
        try:
            if on_startup is not None:
                self.loop.run_until_complete(on_startup())

            if asyncio_debug_mode:
                self.loop.set_debug(True)
            if auto_reload:
                self.loop.create_task(_auto_reload())

            [self.loop.create_task(task) for task in self.tasks]

            logger.info("Loop started!")
            self.loop.run_forever()

        finally:
            if on_shutdown is not None:
                self.loop.run_until_complete(on_shutdown())

    def close(self):
        """
        Close event loop manually

        >>> task_manager.close()
        :return:
        """
        self.loop.close()

    def add_task(self, task: typing.Union[typing.Coroutine, typing.Callable]):
        """

        Add task to loop when loop don`t started.

        :param task: coroutine for run in loop
        :return:
        """
        if asyncio.iscoroutinefunction(task):
            self.tasks.append(task())
            logger.info(f"Task {task.__name__} successfully added!")
        elif asyncio.iscoroutine(task):
            self.tasks.append(task)
        else:
            raise RuntimeError(
                "Unexpected task. Tasks may be only coroutine functions"
            )

    def run_task(self, task: typing.Callable):
        """
        Create task in loop

        >>> async def other_coro():
        >>>    while True:
        >>>        print("hello, my friend!")
        >>>        await asyncio.sleep(5)

        >>> async def my_pretty_coro():
        >>>    task_manager.run_task(other_coro)
        >>>    return True


        :param task:
        :return:
        """

        self.loop.create_task(task())



from enum import Enum

import pydantic

from . import events_objects as EventsObjects
from .events_list import Event as events
from vk.types.attachments import Audio
from vk.types.attachments import Photo
from vk.types.attachments import Video
from vk.types.base import BaseModel
from vk.types.message import Message
from vk.types.wall_post import WallPost

# https://vk.com/dev/groups_events


class BaseEvent(BaseModel):
    group_id: int = None
    type: Enum = None
    object: BaseModel


class MessageNew(BaseEvent):
    type: str = None
    object: EventsObjects.MessageNew = None


class MessageReply(MessageNew):
    type: str = None


class MessageAllow(BaseEvent):
    type: str = None
    object: EventsObjects.MessageAllow = None


class MessageDeny(BaseEvent):
    type: str = None
    object: EventsObjects.MessageAllow = None


class PhotoNew(BaseEvent):
    type: str = None
    object: Photo = None


class PhotoCommentNew(BaseEvent):
    type: str = None
    object: EventsObjects.PhotoCommentNew = None


class PhotoCommentEdit(PhotoCommentNew):
    type: str = None


class PhotoCommentRestore(PhotoCommentNew):
    type: str = None


class PhotoCommentDelete(BaseEvent):
    type: str = None
    object: EventsObjects.PhotoCommentDelete = None


class AudioNew(BaseEvent):
    type: str = None
    object: Audio = None


class VideoNew(BaseEvent):
    type: str = None
    object: Video = None


class VideoCommentNew(BaseEvent):
    type: str = None
    object: EventsObjects.VideoCommentNew = None


class VideoCommentEdit(VideoCommentNew):
    type: str = None


class VideoCommentRestore(VideoCommentNew):
    type: str = None


class VideoCommentDelete(BaseEvent):
    type: str = None
    object: EventsObjects.VideoCommentDelete = None


class WallPostNew(BaseEvent):
    type: str = None
    object: WallPost = None


class WallRepost(WallPostNew):
    type: str = None


class WallReplyNew(BaseEvent):
    type: str = None
    object: EventsObjects.WallReplyNew = None


class WallReplyEdit(WallReplyNew):
    type: str = None


class WallReplyRestore(WallReplyNew):
    type: str = None


class WallReplyDelete(BaseEvent):
    type: str = None
    object: EventsObjects.WallReplyDelete = None


class BoardPostNew(BaseEvent):
    type: str = None
    object: EventsObjects.BoardPostNew = None


class BoardPostEdit(BoardPostNew):
    type: str = None


class BoardPostRestore(BoardPostNew):
    type: str = None


class BoardPostDelete(BaseEvent):
    type: str = None
    object: EventsObjects.BoardPostDelete = None


class MarketCommentNew(BaseEvent):
    type: str = None
    object: EventsObjects.MarketCommentNew = None


class MarketCommentEdit(MarketCommentNew):
    type: str = None


class MarketCommentRestore(MarketCommentNew):
    type: str = None


class MarketCommentDelete(BaseEvent):
    type: str = None
    object: EventsObjects.MarketCommentDelete = None


class GroupLeave(BaseEvent):
    type: str = None
    object: EventsObjects.GroupLeave = None


class GroupJoin(BaseEvent):
    type: str = None
    object: EventsObjects.GroupJoin = None


class UserBlock(BaseEvent):
    type: str = None
    object: EventsObjects.UserBlock = None


class UserUnblock(BaseEvent):
    type: str = None
    object: EventsObjects.UserUnblock = None


class PollVoteNew(BaseEvent):
    type: str = None
    object: EventsObjects.PollVoteNew = None


class GroupOfficersEdit(BaseEvent):
    type: str = None
    object: EventsObjects.GroupOfficersEdit = None


class GroupChangeSettings(BaseEvent):
    type: str = None
    object: EventsObjects.GroupChangeSettings = None


class GroupChangePhoto(BaseEvent):
    type: str = None
    object: EventsObjects.GroupChangePhoto = None


Audio.update_forward_refs()
AudioNew.update_forward_refs()
BaseEvent.update_forward_refs()
BaseModel.update_forward_refs()
BoardPostDelete.update_forward_refs()
BoardPostEdit.update_forward_refs()
BoardPostNew.update_forward_refs()
BoardPostRestore.update_forward_refs()
GroupChangePhoto.update_forward_refs()
GroupChangeSettings.update_forward_refs()
GroupJoin.update_forward_refs()
GroupLeave.update_forward_refs()
GroupOfficersEdit.update_forward_refs()
MarketCommentDelete.update_forward_refs()
MarketCommentEdit.update_forward_refs()
MarketCommentNew.update_forward_refs()
MarketCommentRestore.update_forward_refs()
Message.update_forward_refs()
MessageAllow.update_forward_refs()
MessageDeny.update_forward_refs()
MessageNew.update_forward_refs()
MessageReply.update_forward_refs()
Photo.update_forward_refs()
PhotoCommentDelete.update_forward_refs()
PhotoCommentEdit.update_forward_refs()
PhotoCommentNew.update_forward_refs()
PhotoCommentRestore.update_forward_refs()
PhotoNew.update_forward_refs()
PollVoteNew.update_forward_refs()
UserBlock.update_forward_refs()
UserUnblock.update_forward_refs()
Video.update_forward_refs()
VideoCommentDelete.update_forward_refs()
VideoCommentEdit.update_forward_refs()
VideoCommentNew.update_forward_refs()
VideoCommentRestore.update_forward_refs()
VideoNew.update_forward_refs()
WallPost.update_forward_refs()
WallPostNew.update_forward_refs()
WallReplyDelete.update_forward_refs()
WallReplyEdit.update_forward_refs()
WallReplyNew.update_forward_refs()
WallReplyRestore.update_forward_refs()
WallRepost.update_forward_refs()



from .others import SimpleResponse
from ..base import BaseModel


import typing


class CreateResponse(BaseModel):
    form_id: int = None
    url: str = None


class Create(BaseModel):
    response: CreateResponse = None


class Delete(SimpleResponse):
    pass


class Get(SimpleResponse):
    pass


class GetLeads(BaseModel):
    response: typing.Any = None


class GetUploadURL(BaseModel):
    response: str = None


class List(BaseModel):
    response: typing.List = None


class Update(SimpleResponse):
    pass



import logging
import typing
from abc import ABC

from .handler import SkipHandler
from vk.types import BaseEvent
from vk.utils.mixins import MetaMixin

logger = logging.getLogger(__name__)

if typing.TYPE_CHECKING:
    from .dispatcher import Dispatcher


class MiddlewareManager:
    def __init__(self, dispatcher: "Dispatcher"):
        self.dp: "Dispatcher" = dispatcher
        self.middlewares: typing.List[BaseMiddleware] = []

    def setup(self, middleware: "BaseMiddleware"):
        if not isinstance(middleware, BaseMiddleware):
            raise RuntimeError(
                "Middleware must be only instance of 'BaseMiddleware"
            )

        if middleware.is_configured():
            raise RuntimeError("Middleware already configured!")

        if middleware.meta and middleware.meta.get("deprecated", False):
            logger.warning(
                f"This middleware (({middleware.__class__.__name__})) deprecated. Not recommended to use."
            )

        self.middlewares.append(middleware)
        logger.info(
            f"Middleware '{middleware.__class__.__name__}' successfully added!"
        )

    async def trigger_pre_process_middlewares(
        self, event: BaseEvent, data: dict
    ):
        _skip_handler = False
        for middleware in self.middlewares:
            try:
                has = getattr(middleware, "pre_process_event", None)
                if has is None:
                    continue
                data = await middleware.pre_process_event(event, data)
            except SkipHandler:
                logger.debug(
                    f"Middleware {middleware.__class__.__name__} skip handler!"
                )
                _skip_handler = True
                break  # skip other middlewares if middleware skips handler

        return _skip_handler, data

    async def trigger_post_process_middlewares(self, result: typing.Any):
        """

        :param result: result of handler work
        :return:
        """
        for middleware in self.middlewares:
            has = getattr(middleware, "post_process_event", None)
            if has is None:
                continue
            try:
                await middleware.post_process_event(result)
            except SkipHandler:
                logger.debug(
                    f"Middleware {middleware.__class__.__name__} skip handler!"
                )
                break  # skip other middlewares if middleware skips handler


class AbstractMiddleware(ABC, MetaMixin):
    possible_hooks = ["pre_process_event", "post_process_event"]

    # you should override hooks.

    async def pre_process_event(self, event: BaseEvent, data: dict) -> dict:
        """
        Called before check filters and execute handler
        :param self:
        :param event:
        :param data:
        :return: data
        """

    async def post_process_event(self, result: typing.Any) -> None:
        """
        It is called after handler
        :return:
        """


class BaseMiddleware(AbstractMiddleware, ABC):
    def __init__(self):
        self._configured: bool = False

    def is_configured(self) -> bool:
        return self._configured



from ..base import BaseModel


# https://vk.com/dev/objects/video


class Video(BaseModel):
    id: int = None
    owner_id: int = None
    title: str = None
    description: str = None
    duration: int = None
    photo_130: str = None
    photo_320: str = None
    photo_640: str = None
    photo_800: str = None
    date: int = None
    adding_date: int = None
    views: int = None
    comments: int = None
    player: str = None
    access_key: str = None
    processing: int = None
    live: int = None
    upcoming: int = None
    is_favorite: bool = None



from .photo_sizes import PhotoSizes
from .post_source import PostSource
from .audio_genres import AudioGenres
from .city import City
from .country import Country
from .place import Place, GeoPlace
from .join_type import JoinType
from .block_reason import BlockReason
from .admin_level import AdminLevel
from .active_offer import ActiveOffer
from .sex import Sex
from .bdate import BdateVisiblity
from .country import Country
from .name_request import NameRequest
from .metro_station import MetroStation
from .email import Email



"""
A file which contains all project constants.
"""
from vk.utils.json import AbstractJsonLibrary
from vk.utils.json import JsonLibrary

API_VERSION: str = "5.103"  # current api version https://vk.com/dev/versions
API_LINK: str = "https://api.vk.com/method/"  # link to access API

try:
    import orjson  # noqa
    from orjson import JSONDecodeError as _JSONDecodeError_orjson
except ImportError:
    orjson = None
    _JSONDecodeError_orjson = None

try:
    import ujson  # noqa

    _JSONDecodeError_ujson = ValueError
except ImportError:
    ujson = None
    _JSONDecodeError_ujson = None

if not (ujson or orjson):
    import json
    from json import JSONDecodeError as _JSONDecodeError_json
else:
    json = None
    _JSONDecodeError_json = None

_json_decode_errors = [
    _JSONDecodeError_json,
    _JSONDecodeError_orjson,
    _JSONDecodeError_ujson,
]

_JSONLIB: AbstractJsonLibrary = [lib for lib in [orjson, ujson, json] if lib][
    0
]  # noqa
JSON_LIBRARY = JsonLibrary(_JSONLIB)
del _JSONLIB

JSONDecodeError = tuple([error for error in _json_decode_errors if error])
del _json_decode_errors


def default_rules() -> dict:
    """
    Build and return dict of default handler rules.
    :return:
    """
    from vk.bot_framework.rules.rules import (
        Commands,
        Text,
        Payload,
        ChatAction,
        DataCheck,
        MessageCountArgs,
        MessageArgsValidate,
        InPersonalMessages,
        InChat,
        FromBot,
        WithReplyMessage,
        WithFwdMessages,
        CountFwdMessages,
        Regex,
        VBML,
    )

    _default_rules: dict = {
        "commands": Commands,
        "text": Text,
        "payload": Payload,
        "chat_action": ChatAction,
        "data_check": DataCheck,
        "count_args": MessageCountArgs,
        "have_args": MessageArgsValidate,
        "in_chat": InChat,
        "in_pm": InPersonalMessages,
        "from_bot": FromBot,
        "with_reply_message": WithReplyMessage,
        "with_fwd_messages": WithFwdMessages,
        "count_fwd_messages": CountFwdMessages,
        "regex": Regex,
        "vbml": VBML,
    }
    return _default_rules


def default_extensions() -> dict:
    """
    Build and return dict of default dispatcher extensions
    :return:
    """
    from vk.bot_framework.extensions import Polling

    _default_extensions: dict = {"polling": Polling}

    return _default_extensions



import typing

from ..additional import ActiveOffer
from ..additional import BdateVisiblity
from ..additional import City
from ..additional import Country
from ..additional import NameRequest
from ..additional import Sex
from ..base import BaseModel
from ..community import Community
from ..user import User
from ..user import UserRelation
from .others import SimpleResponse


class Ban(SimpleResponse):
    pass


class ChangePasswordResponse(BaseModel):
    token: str = None
    secret: str = None


class ChangePassword(BaseModel):
    response: ChangePasswordResponse = None


class GetActiveOffersResponse(BaseModel):
    count: int = None
    items: typing.List[ActiveOffer] = []


class GetActiveOffers(BaseModel):
    response: GetActiveOffersResponse = None


class GetAppPermissions(SimpleResponse):
    pass


class GetBannedResponse(BaseModel):
    count: int = None
    items: typing.List[int] = None
    profiles: typing.List[User] = None
    groups: typing.List[Community] = None


class GetBanned(BaseModel):
    response: GetBannedResponse = None


class GetCountersResponse(BaseModel):
    friends: int = None
    friends_suggestions: int = None
    messages: int = None
    photos: int = None
    videos: int = None
    gifts: int = None
    events: int = None
    groups: int = None
    sdk: int = None
    app_requests: int = None


class GetCounters(BaseModel):
    response: GetCountersResponse = None


class GetInfoResponse(BaseModel):
    country: str = None
    https_required: int = None
    _2fa_required: int = None
    own_posts_default: int = None
    no_wall_replies: int = None
    intro: int = None
    lang: int = None


class GetInfo(BaseModel):
    response: GetInfoResponse = None


class GetProfileInfoResponse(BaseModel):
    first_name: str = None
    last_name: str = None
    maiden_name: str = None
    screen_name: str = None
    sex: Sex = None
    relation: UserRelation = None
    relation_partner: User = None
    relation_pending: int = None
    relation_requests: typing.List[User] = None
    bdate: str = None
    bdate_visiblity: BdateVisiblity = None
    home_town: str = None
    country: Country = None
    city: City = None
    name_request: NameRequest = None
    status: str = None
    phone: str = None


class GetProfileInfo(BaseModel):
    response: GetProfileInfoResponse = None


class GetPushSettingsResponse(BaseModel):
    disabled: int = None
    disabled_until: int = None
    conversations: list = None
    settings: typing.Any = None


class GetPushSettings(BaseModel):
    response: GetPushSettingsResponse = None


class RegisterDevice(SimpleResponse):
    pass


class SaveProfileInfoResponse(BaseModel):
    changed: int = None
    name_request: NameRequest = None


class SaveProfileInfo(BaseModel):
    response: SaveProfileInfoResponse = None


class SetInfo(SimpleResponse):
    pass


class SetNameInMenu(SimpleResponse):
    pass


class SetOffline(SimpleResponse):
    pass


class SetOnline(SimpleResponse):
    pass


class SetPushSettings(SimpleResponse):
    pass


class SetSilenceMode(SimpleResponse):
    pass


class Unban(SimpleResponse):
    pass


class UnregisterDevice(SimpleResponse):
    pass



import asyncio
import typing

from ..dispatcher.storage import AbstractAsyncExpiredStorage

try:
    import aioredis  # noqa
except ImportError:
    aioredis = None


class RedisStorage(AbstractAsyncExpiredStorage):
    def __init__(
        self,
        address: str,
        loop: asyncio.AbstractEventLoop,
        db: str = None,
        password: str = None,
    ):
        if not aioredis:
            raise RuntimeError(
                "For use this storage install aioredis (pip install aioredis)"
            )
        self._address = address
        self._db = db
        self._password = password
        self._loop = loop
        self.connection: typing.Optional["aioredis.Redis"] = None

    async def create_connection(self) -> "aioredis.Redis":
        if not self.connection:
            conn: aioredis.Redis = await aioredis.create_redis(
                address=self._address,
                db=self._db,
                password=self._password,
                loop=self._loop,
            )
            self.connection = conn
            return conn
        else:
            raise RuntimeError("Connection already setuped")

    async def place(
        self, key: typing.AnyStr, value: typing.Any, expire=0, pexpire=0
    ) -> None:
        if not self.connection:
            await self.create_connection()
        await self.connection.set(key, value, expire=expire, pexpire=pexpire)

    async def get(
        self, key: typing.AnyStr, default: typing.Any = None
    ) -> typing.Optional[typing.Any]:
        if not self.connection:
            await self.create_connection()
        value = await self.connection.get(key)
        if value is None:
            return default
        return value.decode()

    async def update(
        self, key: typing.AnyStr, value: typing.Any, expire=0, pexpire=0
    ):
        if not self.connection:
            await self.create_connection()
        await self.place(key, value, expire, pexpire)

    async def delete(
        self, key: typing.AnyStr, *keys: typing.Tuple[typing.AnyStr]
    ) -> None:
        if not self.connection:
            await self.create_connection()
        await self.connection.delete(key, *keys)

    async def exists(
        self, key: typing.AnyStr, *keys: typing.Tuple[typing.AnyStr]
    ) -> bool:
        if not self.connection:
            await self.create_connection()
        result = await self.connection.exists(key, *keys)
        if result == 0:
            return False
        else:
            return True



from .base import BaseMethod
from vk.types.responses import apps as m


class Apps(BaseMethod):
    async def delete_app_requests(self,):
        """
        Delete all request notifications from the current app.


        """
        method = self.get_method_name(self.delete_app_requests)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DeleteAppRequests(**r)

    async def get(
        self,
        app_id: int = None,
        app_ids: list = None,
        platform: str = None,
        extended: bool = None,
        return_friends: bool = None,
        fields: list = None,
        name_case: str = None,
    ):
        """
        Return applications data.
        :param app_id: Application ID
        :param app_ids: List of application ID
        :param platform: platform. Possible values: *'ios' — iOS,, *'android' — Android,, *'winphone' — Windows Phone,, *'web' — приложения на vk.com. By default: 'web'.
        :param extended:
        :param return_friends:
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'contacts', 'education', 'online', 'counters', 'relation', 'last_seen', 'activity', 'can_write_private_message', 'can_see_all_posts', 'can_post', 'universities', (only if return_friends - 1)
        :param name_case: Case for declension of user name and surname: 'nom' — nominative (default),, 'gen' — genitive,, 'dat' — dative,, 'acc' — accusative,, 'ins' — instrumental,, 'abl' — prepositional. (only if 'return_friends' = '1')


        """
        method = self.get_method_name(self.get)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Get(**r)

    async def get_catalog(
        self,
        sort: str = None,
        offset: int = None,
        count: int = None,
        platform: str = None,
        extended: bool = None,
        return_friends: bool = None,
        fields: list = None,
        name_case: str = None,
        q: str = None,
        genre_id: int = None,
        filter: str = None,
    ):
        """
        Return a list of applications (apps) available to users in the App Catalog.
        :param sort: Sort order: 'popular_today' — popular for one day (default), 'visitors' — by visitors number , 'create_date' — by creation date, 'growth_rate' — by growth rate, 'popular_week' — popular for one week
        :param offset: Offset required to return a specific subset of apps.
        :param count: Number of apps to return.
        :param platform:
        :param extended: '1' — to return additional fields 'screenshots', 'MAU', 'catalog_position', and 'international'. If set, 'count' must be less than or equal to '100'. '0' — not to return additional fields (default).
        :param return_friends:
        :param fields:
        :param name_case:
        :param q: Search query string.
        :param genre_id:
        :param filter: 'installed' — to return list of installed apps (only for mobile platform).


        """
        method = self.get_method_name(self.get_catalog)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCatalog(**r)

    async def get_friends_list(
        self,
        extended: bool = None,
        count: int = None,
        offset: int = None,
        type: str = None,
        fields: list = None,
    ):
        """
        Create friends list for requests and invites in current app.
        :param extended:
        :param count: List size.
        :param offset:
        :param type: List type. Possible values: * 'invite' — available for invites (don't play the game),, * 'request' — available for request (play the game). By default: 'invite'.
        :param fields: Additional profile fields, see [vk.com/dev/fields|description].


        """
        method = self.get_method_name(self.get_friends_list)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetFriendsList(**r)

    async def get_leaderboard(
        self, type: str = None, global_: bool = None, extended: bool = None
    ):
        """
        Return players rating in the game.
        :param type: Leaderboard type. Possible values: *'level' — by level,, *'points' — by mission points,, *'score' — by score ().
        :param global_: Rating type. Possible values: *'1' — global rating among all players,, *'0' — rating among user friends.
        :param extended: 1 — to return additional info about users


        """
        method = self.get_method_name(self.get_leaderboard)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetLeaderboard(**r)

    async def get_scopes(self, type: str = None):
        """
        Return scopes for auth
        :param type:


        """
        method = self.get_method_name(self.get_scopes)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetScopes(**r)

    async def get_score(self, user_id: int = None):
        """
        Return user score in app
        :param user_id:


        """
        method = self.get_method_name(self.get_score)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetScore(**r)

    async def send_request(
        self,
        user_id: int = None,
        text: str = None,
        type: str = None,
        name: str = None,
        key: str = None,
        separate: bool = None,
    ):
        """
        Send a request to the another user in the app that uses VK authorization.
        :param user_id: id of the user to send a request
        :param text: request text
        :param type: request type. Values: 'invite' – if the request is sent to a user who does not have the app installed,, 'request' – if a user has already installed the app
        :param name:
        :param key: special string key to be sent with the request
        :param separate:


        """
        method = self.get_method_name(self.send_request)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SendRequest(**r)



from .base import BaseMethod
from vk.types.responses import fave as m


class Fave(BaseMethod):
    async def add_article(self, url: str = None):
        """

        :param url:


        """
        method = self.get_method_name(self.add_article)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddArticle(**r)

    async def add_link(self, link: str = None):
        """
        Add a link to the user faves.
        :param link: Link URL.


        """
        method = self.get_method_name(self.add_link)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddLink(**r)

    async def add_page(self, user_id: int = None, group_id: int = None):
        """

        :param user_id:
        :param group_id:


        """
        method = self.get_method_name(self.add_page)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddPage(**r)

    async def add_post(
        self, owner_id: int = None, id: int = None, access_key: str = None
    ):
        """

        :param owner_id:
        :param id:
        :param access_key:


        """
        method = self.get_method_name(self.add_post)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddPost(**r)

    async def add_product(
        self, owner_id: int = None, id: int = None, access_key: str = None
    ):
        """

        :param owner_id:
        :param id:
        :param access_key:


        """
        method = self.get_method_name(self.add_product)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddProduct(**r)

    async def add_tag(self, name: str = None):
        """

        :param name:


        """
        method = self.get_method_name(self.add_tag)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddTag(**r)

    async def add_video(
        self, owner_id: int = None, id: int = None, access_key: str = None
    ):
        """

        :param owner_id:
        :param id:
        :param access_key:


        """
        method = self.get_method_name(self.add_video)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddVideo(**r)

    async def edit_tag(self, id: int = None, name: str = None):
        """

        :param id:
        :param name:


        """
        method = self.get_method_name(self.edit_tag)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.EditTag(**r)

    async def get(
        self,
        extended: bool = None,
        item_type: str = None,
        tag_id: int = None,
        offset: int = None,
        count: int = None,
        fields: str = None,
        is_from_snackbar: bool = None,
    ):
        """

        :param extended: '1' — to return additional 'wall', 'profiles', and 'groups' fields. By default: '0'.
        :param item_type:
        :param tag_id: Tag ID.
        :param offset: Offset needed to return a specific subset of users.
        :param count: Number of users to return.
        :param fields:
        :param is_from_snackbar:


        """
        method = self.get_method_name(self.get)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Get(**r)

    async def get_pages(
        self,
        offset: int = None,
        count: int = None,
        type: str = None,
        fields: list = None,
        tag_id: int = None,
    ):
        """

        :param offset:
        :param count:
        :param type:
        :param fields:
        :param tag_id:


        """
        method = self.get_method_name(self.get_pages)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetPages(**r)

    async def get_tags(self,):
        """



        """
        method = self.get_method_name(self.get_tags)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetTags(**r)

    async def mark_seen(self,):
        """



        """
        method = self.get_method_name(self.mark_seen)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MarkSeen(**r)

    async def remove_article(
        self, owner_id: int = None, article_id: int = None
    ):
        """

        :param owner_id:
        :param article_id:


        """
        method = self.get_method_name(self.remove_article)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveArticle(**r)

    async def remove_link(self, link_id: str = None, link: str = None):
        """
        Removes link from the user's faves.
        :param link_id: Link ID (can be obtained by [vk.com/dev/faves.getLinks|faves.getLinks] method).
        :param link: Link URL


        """
        method = self.get_method_name(self.remove_link)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveLink(**r)

    async def remove_page(self, user_id: int = None, group_id: int = None):
        """

        :param user_id:
        :param group_id:


        """
        method = self.get_method_name(self.remove_page)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemovePage(**r)

    async def remove_post(self, owner_id: int = None, id: int = None):
        """

        :param owner_id:
        :param id:


        """
        method = self.get_method_name(self.remove_post)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemovePost(**r)

    async def remove_product(self, owner_id: int = None, id: int = None):
        """

        :param owner_id:
        :param id:


        """
        method = self.get_method_name(self.remove_product)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveProduct(**r)

    async def remove_tag(self, id: int = None):
        """

        :param id:


        """
        method = self.get_method_name(self.remove_tag)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveTag(**r)

    async def reorder_tags(self, ids: list = None):
        """

        :param ids:


        """
        method = self.get_method_name(self.reorder_tags)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.ReorderTags(**r)

    async def set_page_tags(
        self, user_id: int = None, group_id: int = None, tag_ids: list = None
    ):
        """

        :param user_id:
        :param group_id:
        :param tag_ids:


        """
        method = self.get_method_name(self.set_page_tags)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetPageTags(**r)

    async def set_tags(
        self,
        item_type: str = None,
        item_owner_id: int = None,
        item_id: int = None,
        tag_ids: list = None,
        link_id: str = None,
        link_url: str = None,
    ):
        """

        :param item_type:
        :param item_owner_id:
        :param item_id:
        :param tag_ids:
        :param link_id:
        :param link_url:


        """
        method = self.get_method_name(self.set_tags)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetTags(**r)

    async def track_page_interaction(
        self, user_id: int = None, group_id: int = None
    ):
        """

        :param user_id:
        :param group_id:


        """
        method = self.get_method_name(self.track_page_interaction)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.TrackPageInteraction(**r)



from .base import BaseMethod
from vk.types.responses import database as m


class Database(BaseMethod):
    async def get_chairs(
        self, faculty_id: int = None, offset: int = None, count: int = None
    ):
        """
        Return list of chairs on a specified faculty.
        :param faculty_id: id of the faculty to get chairs from
        :param offset: offset required to get a certain subset of chairs
        :param count: amount of chairs to get


        """
        method = self.get_method_name(self.get_chairs)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetChairs(**r)

    async def get_cities(
        self,
        country_id: int = None,
        region_id: int = None,
        q: str = None,
        need_all: bool = None,
        offset: int = None,
        count: int = None,
    ):
        """
        Return a list of cities.
        :param country_id: Country ID.
        :param region_id: Region ID.
        :param q: Search query.
        :param need_all: '1' — to return all cities in the country, '0' — to return major cities in the country (default),
        :param offset: Offset needed to return a specific subset of cities.
        :param count: Number of cities to return.


        """
        method = self.get_method_name(self.get_cities)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCities(**r)

    async def get_cities_by_id(self, city_ids: list = None):
        """
        Return information about cities by their IDs.
        :param city_ids: City IDs.


        """
        method = self.get_method_name(self.get_cities_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCitiesById(**r)

    async def get_countries(
        self,
        need_all: bool = None,
        code: str = None,
        offset: int = None,
        count: int = None,
    ):
        """
        Return a list of countries.
        :param need_all: '1' — to return a full list of all countries, '0' — to return a list of countries near the current user's country (default).
        :param code: Country codes in [vk.com/dev/country_codes|ISO 3166-1 alpha-2] standard.
        :param offset: Offset needed to return a specific subset of countries.
        :param count: Number of countries to return.


        """
        method = self.get_method_name(self.get_countries)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCountries(**r)

    async def get_countries_by_id(self, country_ids: list = None):
        """
        Return information about countries by their IDs.
        :param country_ids: Country IDs.


        """
        method = self.get_method_name(self.get_countries_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCountriesById(**r)

    async def get_faculties(
        self, university_id: int = None, offset: int = None, count: int = None
    ):
        """
        Return a list of faculties (i.e., university departments).
        :param university_id: University ID.
        :param offset: Offset needed to return a specific subset of faculties.
        :param count: Number of faculties to return.


        """
        method = self.get_method_name(self.get_faculties)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetFaculties(**r)

    async def get_metro_stations(
        self,
        city_id: int = None,
        offset: int = None,
        count: int = None,
        extended: bool = None,
    ):
        """
        Get metro stations by city
        :param city_id:
        :param offset:
        :param count:
        :param extended:


        """
        method = self.get_method_name(self.get_metro_stations)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetMetroStations(**r)

    async def get_metro_stations_by_id(self, station_ids: list = None):
        """
        Get metro station by his id
        :param station_ids:


        """
        method = self.get_method_name(self.get_metro_stations_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetMetroStationsById(**r)

    async def get_regions(
        self,
        country_id: int = None,
        q: str = None,
        offset: int = None,
        count: int = None,
    ):
        """
        Return a list of regions.
        :param country_id: Country ID, received in [vk.com/dev/database.getCountries|database.getCountries] method.
        :param q: Search query.
        :param offset: Offset needed to return specific subset of regions.
        :param count: Number of regions to return.


        """
        method = self.get_method_name(self.get_regions)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetRegions(**r)

    async def get_school_classes(self, country_id: int = None):
        """
        Return a list of school classes specified for the country.
        :param country_id: Country ID.


        """
        method = self.get_method_name(self.get_school_classes)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetSchoolClasses(**r)

    async def get_schools(
        self,
        q: str = None,
        city_id: int = None,
        offset: int = None,
        count: int = None,
    ):
        """
        Return a list of schools.
        :param q: Search query.
        :param city_id: City ID.
        :param offset: Offset needed to return a specific subset of schools.
        :param count: Number of schools to return.


        """
        method = self.get_method_name(self.get_schools)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetSchools(**r)

    async def get_universities(
        self,
        q: str = None,
        country_id: int = None,
        city_id: int = None,
        offset: int = None,
        count: int = None,
    ):
        """
        Return a list of higher education institutions.
        :param q: Search query.
        :param country_id: Country ID.
        :param city_id: City ID.
        :param offset: Offset needed to return a specific subset of universities.
        :param count: Number of universities to return.


        """
        method = self.get_method_name(self.get_universities)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetUniversities(**r)



import ast

from ..converter import VKScriptConverter


@VKScriptConverter.register(ast.Dict)
def dict_handler(node: ast.Dict):
    converter = VKScriptConverter.get_current()
    inner = ",".join(
        f"{converter.convert_node(key)}:{converter.convert_node(value)}"
        for key, value in zip(node.keys, node.values)
    )
    return f"{{{inner}}}"


@VKScriptConverter.register(ast.List)
def list_handler(node: ast.List):
    converter = VKScriptConverter.get_current()
    return f'[{",".join(converter.convert_node(element) for element in node.elts)}]'


@VKScriptConverter.register(ast.Tuple)
def tuple_handler(node: ast.Tuple):
    converter = VKScriptConverter.get_current()
    return f'[{",".join(converter.convert_node(element) for element in node.elts)}]'


@VKScriptConverter.register(ast.Num)
def num_handler(node: ast.Num):
    return repr(node.n)


@VKScriptConverter.register(ast.Str)
def str_handler(node: ast.Str):
    return repr(node.s)


@VKScriptConverter.register(ast.NameConstant)
def name_constant_handler(node: ast.NameConstant):
    constants = {None: "null", True: "true", False: "false"}
    if node.value not in constants:
        raise NotImplementedError(f"constant {node.value} not implemented")
    return constants[node.value]



from enum import IntEnum

from ..base import BaseModel
from vk.types.additional import PhotoSizes
from vk.types.attachments import AudioMsg
from vk.types.attachments import Graffiti


# https://vk.com/dev/objects/doc


class DocumentType(IntEnum):
    text_document = 1
    archive = 2
    gif = 3
    image = 4
    audio = 5
    video = 6
    ebooks = 7
    unknown = 8


class DocumentPreview(BaseModel):
    photo: PhotoSizes = None
    graffiti: Graffiti = None
    audio_msg: AudioMsg = None


class Document(BaseModel):
    id: int = None
    owner_id: int = None
    title: str = None
    size: int = None
    ext: str = None
    url: str = None
    date: int = None
    type: DocumentType = None
    preview: DocumentPreview = None
    access_key: str = None



import typing

import pydantic

from vk.utils.mixins import ContextInstanceMixin


class Scope(pydantic.BaseModel):
    locals: list = []
    globals: dict = {}


class VKScriptConverter(ContextInstanceMixin):
    handlers: dict = {}

    @classmethod
    def register(cls, expr):
        def meta(handler: typing.Callable):
            cls.handlers[expr] = handler

        return meta

    def __init__(self, scope: Scope = None):
        self.scope = scope or Scope()
        self.set_current(self)

    def convert_node(self, node):
        if node.__class__ in self.handlers:
            return self.handlers[node.__class__](node)
        raise NotImplementedError(
            f"Conversion for type {node.__class__} not implemented."
        )

    def convert_block(self, nodes: list):
        return "".join(self.convert_node(child) for child in nodes)



from .base import BaseMethod
from vk.types.responses import account as m


class Account(BaseMethod):
    async def ban(self, owner_id: int = None):
        """

        :param owner_id:


        """
        method = self.get_method_name(self.ban)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Ban(**r)

    async def change_password(
        self,
        restore_sid: str = None,
        change_password_hash: str = None,
        old_password: str = None,
        new_password: str = None,
    ):
        """
        Changes a user password after access is successfully restored with the [vk.com/dev/auth.restore|auth.restore] method.
        :param restore_sid: Session id received after the [vk.com/dev/auth.restore|auth.restore] method is executed. (If the password is changed right after the access was restored)
        :param change_password_hash: Hash received after a successful OAuth authorization with a code got by SMS. (If the password is changed right after the access was restored)
        :param old_password: Current user password.
        :param new_password: New password that will be set as a current


        """
        method = self.get_method_name(self.change_password)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.ChangePassword(**r)

    async def get_active_offers(self, offset: int = None, count: int = None):
        """
        Return a list of active ads (offers) which are executed by the user.
        :param offset:
        :param count: Number of results to return.


        """
        method = self.get_method_name(self.get_active_offers)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetActiveOffers(**r)

    async def get_app_permissions(self, user_id: int = None):
        """
        Get settings of the user in this application.
        :param user_id: User ID whose settings information shall be got. By default: current user.


        """
        method = self.get_method_name(self.get_app_permissions)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetAppPermissions(**r)

    async def get_banned(self, offset: int = None, count: int = None):
        """
        Return a user's blacklist.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.


        """
        method = self.get_method_name(self.get_banned)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetBanned(**r)

    async def get_counters(self, filter: list = None):
        """
        Return non-null values of user counters.
        :param filter: Counters to be returned.


        """
        method = self.get_method_name(self.get_counters)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCounters(**r)

    async def get_info(self, fields: list = None):
        """
        Return current account info.
        :param fields: Fields to return. Possible values: *'country' — user country,, *'https_required' — is "HTTPS only" option enabled,, *'own_posts_default' — is "Show my posts only" option is enabled,, *'no_wall_replies' — are wall replies disabled or not,, *'intro' — is intro passed by user or not,, *'lang' — user language. By default: all.


        """
        method = self.get_method_name(self.get_info)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetInfo(**r)

    async def get_profile_info(self,):
        """
        Return the current account info.


        """
        method = self.get_method_name(self.get_profile_info)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetProfileInfo(**r)

    async def get_push_settings(self, device_id: str = None):
        """
        Get settings to push notifications.
        :param device_id: Unique device ID.


        """
        method = self.get_method_name(self.get_push_settings)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetPushSettings(**r)

    async def register_device(
        self,
        token: str = None,
        device_model: str = None,
        device_year: int = None,
        device_id: str = None,
        system_version: str = None,
        settings: str = None,
        sandbox: bool = None,
    ):
        """
        Subscribes an iOS/Android/Windows Phone-based device to receive push notifications
        :param token: Device token used to send notifications. (for mpns, the token shall be URL for sending of notifications)
        :param device_model: String name of device model.
        :param device_year: Device year.
        :param device_id: Unique device ID.
        :param system_version: String version of device operating system.
        :param settings: Push settings in a [vk.com/dev/push_settings|special format].
        :param sandbox:


        """
        method = self.get_method_name(self.register_device)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RegisterDevice(**r)

    async def save_profile_info(
        self,
        first_name: str = None,
        last_name: str = None,
        maiden_name: str = None,
        screen_name: str = None,
        cancel_request_id: int = None,
        sex: int = None,
        relation: int = None,
        relation_partner_id: int = None,
        bdate: str = None,
        bdate_visibility: int = None,
        home_town: str = None,
        country_id: int = None,
        city_id: int = None,
        status: str = None,
    ):
        """
        Edits current profile info.
        :param first_name: User first name.
        :param last_name: User last name.
        :param maiden_name: User maiden name (female only)
        :param screen_name: User screen name.
        :param cancel_request_id: ID of the name change request to be canceled. If this parameter is sent, all the others are ignored.
        :param sex: User sex. Possible values: , * '1' – female,, * '2' – male.
        :param relation: User relationship status. Possible values: , * '1' – single,, * '2' – in a relationship,, * '3' – engaged,, * '4' – married,, * '5' – it's complicated,, * '6' – actively searching,, * '7' – in love,, * '0' – not specified.
        :param relation_partner_id: ID of the relationship partner.
        :param bdate: User birth date, format: DD.MM.YYYY.
        :param bdate_visibility: Birth date visibility. Returned values: , * '1' – show birth date,, * '2' – show only month and day,, * '0' – hide birth date.
        :param home_town: User home town.
        :param country_id: User country.
        :param city_id: User city.
        :param status: Status text.


        """
        method = self.get_method_name(self.save_profile_info)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SaveProfileInfo(**r)

    async def set_info(self, name: str = None, value: str = None):
        """
        Allow to edit the current account info.
        :param name: Setting name.
        :param value: Setting value.


        """
        method = self.get_method_name(self.set_info)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetInfo(**r)

    async def set_name_in_menu(self, user_id: int = None, name: str = None):
        """
        Set an application screen name (up to 17 characters), this is shown in the left menu.
        :param user_id: User ID.
        :param name: Application screen name.


        """
        method = self.get_method_name(self.set_name_in_menu)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetNameInMenu(**r)

    async def set_offline(self,):
        """
        Mark a current user as offline.


        """
        method = self.get_method_name(self.set_offline)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetOffline(**r)

    async def set_online(self, voip: bool = None):
        """
        Mark a current user as online up to 15 minutes.
        :param voip: '1' if videocalls are available for current device.


        """
        method = self.get_method_name(self.set_online)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetOnline(**r)

    async def set_push_settings(
        self,
        device_id: str = None,
        settings: str = None,
        key: str = None,
        value: list = None,
    ):
        """
        Change push settings.
        :param device_id: Unique device ID.
        :param settings: Push settings in a [vk.com/dev/push_settings|special format].
        :param key: Notification key.
        :param value: New value for the key in a [vk.com/dev/push_settings|special format].


        """
        method = self.get_method_name(self.set_push_settings)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetPushSettings(**r)

    async def set_silence_mode(
        self,
        device_id: str = None,
        time: int = None,
        peer_id: int = None,
        sound: int = None,
    ):
        """
        Mute push notifications for the set period of time.
        :param device_id: Unique device ID.
        :param time: Time in seconds for what notifications should be disabled. '-1' to disable forever.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param sound: '1' — to enable sound in this dialog, '0' — to disable sound. Only if 'peer_id' contains user or community ID.


        """
        method = self.get_method_name(self.set_silence_mode)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetSilenceMode(**r)

    async def unban(self, owner_id: int = None):
        """

        :param owner_id:


        """
        method = self.get_method_name(self.unban)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Unban(**r)

    async def unregister_device(
        self, device_id: str = None, sandbox: bool = None
    ):
        """
        Unsubscribe device from push notifications.
        :param device_id: Unique device ID.
        :param sandbox:


        """
        method = self.get_method_name(self.unregister_device)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.UnregisterDevice(**r)



from .others import SimpleResponse
from ..base import BaseModel
from ..attachments.gift import Gift


import typing


class GetResponseItem(BaseModel):
    id: int = None
    from_id: int = None
    message: str = None
    date: int = None
    gift: Gift = None
    privacy: int = None
    gift_hash: str = None


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[GetResponseItem] = None


class Get(BaseModel):
    response: GetResponse = None



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['vk',
 'vk.bot_framework',
 'vk.bot_framework.addons',
 'vk.bot_framework.addons.caching',
 'vk.bot_framework.addons.validators',
 'vk.bot_framework.dispatcher',
 'vk.bot_framework.extensions',
 'vk.bot_framework.middlewares',
 'vk.bot_framework.rules',
 'vk.bot_framework.storages',
 'vk.exceptions',
 'vk.keyboards',
 'vk.longpoll',
 'vk.longpoll.bot',
 'vk.longpoll.user',
 'vk.methods',
 'vk.streaming_api',
 'vk.types',
 'vk.types.additional',
 'vk.types.attachments',
 'vk.types.events',
 'vk.types.events.community',
 'vk.types.responses',
 'vk.utils',
 'vk.utils.vkscript']

package_data = \
{'': ['*'], 'vk.utils.vkscript': ['handlers/*']}

install_requires = \
['aiohttp>=3.6,<4.0', 'pydantic', 'watchgod>=0.5.0,<0.6.0']

extras_require = \
{':python_version == "3.6"': ['async-generator>=1.10,<2.0',
                              'contextvars>=2.4,<3.0'],
 'vbml': ['vbml>=0.5.0,<0.6.0']}

setup_kwargs = {
    'name': 'vk.py-1.0.0a6',
    'version': '1.0.0',
    'description': 'Extremely-fast, easy-to-use, ready for production. The asyncio based library for Python and Humans written to be efficient and reliable.',
    'long_description': 'Reupload vk.py, which the deleted on pypi and github',
    'author': 'Cl0ckHvH',
    'author_email': None,
    'url': 'https://github.com/Cl0ckHvH/vk.py-1.0.0a6',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from vk.types.events.community import event as eventobj
from vk.types.events.community.events_list import Event


def get_event_object(event) -> eventobj:
    _event_type = Event(event["type"])

    if _event_type is Event.MESSAGE_NEW:
        ev = eventobj.MessageNew(**event)

    elif _event_type is Event.MESSAGE_REPLY:
        ev = eventobj.MessageReply(**event)

    elif _event_type is Event.MESSAGE_ALLOW:
        ev = eventobj.MessageAllow(**event)

    elif _event_type is Event.MESSAGES_DENY:
        ev = eventobj.MessageDeny(**event)

    elif _event_type is Event.PHOTO_NEW:
        ev = eventobj.PhotoNew(**event)

    elif _event_type is Event.PHOTO_COMMENT_NEW:
        ev = eventobj.PhotoCommentNew(**event)

    elif _event_type is Event.PHOTO_COMMENT_EDIT:
        ev = eventobj.PhotoCommentEdit(**event)

    elif _event_type is Event.PHOTO_COMMENT_RESTORE:
        ev = eventobj.PhotoCommentRestore(**event)

    elif _event_type is Event.PHOTO_COMMENT_DELETE:
        ev = eventobj.PhotoCommentDelete(**event)

    elif _event_type is Event.AUDIO_NEW:
        ev = eventobj.AudioNew(**event)

    elif _event_type is Event.VIDEO_NEW:
        ev = eventobj.VideoNew(**event)

    elif _event_type is Event.VIDEO_COMMENT_NEW:
        ev = eventobj.VideoCommentNew(**event)

    elif _event_type is Event.VIDEO_COMMENT_EDIT:
        ev = eventobj.VideoCommentEdit(**event)

    elif _event_type is Event.VIDEO_COMMENT_RESTORE:
        ev = eventobj.VideoCommentRestore(**event)

    elif _event_type is Event.VIDEO_COMMENT_DELETE:
        ev = eventobj.VideoCommentDelete(**event)

    elif _event_type is Event.WALL_POST_NEW:
        ev = eventobj.WallPostNew(**event)

    elif _event_type is Event.WALL_REPOST:
        ev = eventobj.WallRepost(**event)

    elif _event_type is Event.WALL_REPLY_NEW:
        ev = eventobj.WallReplyNew(**event)

    elif _event_type is Event.WALL_REPLY_EDIT:
        ev = eventobj.WallReplyEdit(**event)

    elif _event_type is Event.WALL_REPLY_RESTORE:
        ev = eventobj.WallReplyRestore(**event)

    elif _event_type is Event.WALL_REPLY_DELETE:
        ev = eventobj.WallReplyDelete(**event)

    elif _event_type is Event.BOARD_POST_NEW:
        ev = eventobj.BoardPostNew(**event)

    elif _event_type is Event.BOARD_POST_EDIT:
        ev = eventobj.BoardPostEdit(**event)

    elif _event_type is Event.BOARD_POST_RESTORE:
        ev = eventobj.BoardPostRestore(**event)

    elif _event_type is Event.BOARD_POST_DELETE:
        ev = eventobj.BoardPostDelete(**event)

    elif _event_type is Event.MARKET_COMMENT_NEW:
        ev = eventobj.MarketCommentNew(**event)

    elif _event_type is Event.MARKET_COMMENT_EDIT:
        ev = eventobj.MarketCommentEdit(**event)

    elif _event_type is Event.MARKET_COMMENT_RESTORE:
        ev = eventobj.MarketCommentRestore(**event)

    elif _event_type is Event.MARKET_COMMENT_DELETE:
        ev = eventobj.MarketCommentDelete(**event)

    elif _event_type is Event.GROUP_LEAVE:
        ev = eventobj.GroupLeave(**event)

    elif _event_type is Event.GROUP_JOIN:
        ev = eventobj.GroupJoin(**event)

    elif _event_type is Event.USER_BLOCK:
        ev = eventobj.UserBlock(**event)

    elif _event_type is Event.USER_UNBLOCK:
        ev = eventobj.UserUnblock(**event)

    elif _event_type is Event.POLL_VOTE_NEW:
        ev = eventobj.PollVoteNew(**event)

    elif _event_type is Event.GROUP_OFFICERS_EDIT:
        ev = eventobj.GroupOfficersEdit(**event)

    elif _event_type is Event.GROUP_CHANGE_SETTINGS:
        ev = eventobj.GroupChangeSettings(**event)

    elif _event_type is Event.GROUP_CHANGE_PHOTO:
        ev = eventobj.GroupChangePhoto(**event)

    else:
        raise RuntimeError(
            "Unexpected behavior. May be you handle NOT vk event."
        )

    ev.set_current(ev)
    ev.object.set_current(ev.object)
    return ev



from enum import Enum
from ..base import BaseModel
from .photo import Photo, PostedPhoto
from .video import Video
from .audio import Audio
from .document import Document
from .graffiti import Graffiti
from .link import Link
from .note import Note
from .app import App
from .poll import Poll
from .page import Page
from .album import Album
from .market import Market
from .market_album import MarketAlbum
from .sticker import Sticker
from .pretty_cards import PrettyCards
from .audio_msg import AudioMsg

import typing
from typing import Optional


# https://vk.com/dev/objects/attachments_w


class Attachments(str, Enum):
    not_attachments = []
    photo = Photo
    posted_photo = PostedPhoto
    video = Video
    audio = Audio
    document = Document
    graffiti = Graffiti
    link = Link
    note = Note
    app = App
    poll = Poll
    page = Page
    album = Album
    photos_list = typing.List[int]
    market = Market
    market_album = MarketAlbum
    sticker = Sticker
    pretty_cards = PrettyCards


class Attachment(BaseModel):
    type: str = None
    photo: Optional[Photo]
    posted_photo: Optional[PostedPhoto]
    video: Optional[Video]
    audio: Optional[Audio]
    doc: Optional[Document]
    graffiti: Optional[Graffiti]
    link: Optional[Link]
    note: Optional[Note]
    app: Optional[App]
    poll: Optional[Poll]
    page: Optional[Page]
    album: Optional[Album]
    photos_list: Optional[typing.List[Photo]]
    market: Optional[Market]
    market_album: Optional[MarketAlbum]
    sticker: Optional[Sticker]
    pretty_cards: Optional[PrettyCards]
    audio_message: Optional[AudioMsg]



"""
Author: https://github.com/aiogram/aiogram/blob/dev-2.x/aiogram/utils/deprecated.py
"""
import asyncio
import functools
import inspect
import warnings
from typing import Callable


def deprecated(reason, stacklevel=2) -> Callable:
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.
    Source: https://stackoverflow.com/questions/2536307/decorators-in-the-python-standard-lib-deprecated-specifically
    """

    if isinstance(reason, str):

        # The @deprecated is used with a 'reason'.
        #
        # .. code-block:: python
        #
        #    @deprecated("please, use another function")
        #    def old_function(x, y):
        #      pass

        def decorator(func):

            if inspect.isclass(func):
                msg = "Call to deprecated class {name} ({reason})."
            else:
                msg = "Call to deprecated function {name} ({reason})."

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                warn_deprecated(
                    msg.format(name=func.__name__, reason=reason),
                    stacklevel=stacklevel,
                )
                warnings.simplefilter("default", DeprecationWarning)
                return func(*args, **kwargs)

            return wrapper

        return decorator

    if inspect.isclass(reason) or inspect.isfunction(reason):

        # The @deprecated is used without any 'reason'.
        #
        # .. code-block:: python
        #
        #    @deprecated
        #    def old_function(x, y):
        #      pass

        func1 = reason

        if inspect.isclass(func1):
            msg1 = "Call to deprecated class {name}."
        else:
            msg1 = "Call to deprecated function {name}."

        @functools.wraps(func1)
        def wrapper1(*args, **kwargs):
            warn_deprecated(
                msg1.format(name=func1.__name__), stacklevel=stacklevel
            )
            return func1(*args, **kwargs)

        return wrapper1

    raise TypeError(repr(type(reason)))


def warn_deprecated(message, warning=DeprecationWarning, stacklevel=2):
    warnings.simplefilter("always", warning)
    warnings.warn(message, category=warning, stacklevel=stacklevel)
    warnings.simplefilter("default", warning)


def renamed_argument(
    old_name: str, new_name: str, until_version: str, stacklevel: int = 3
):
    """
    A meta-decorator to mark an argument as deprecated.
    .. code-block:: python3
        @renamed_argument("chat", "chat_id", "3.0")  # stacklevel=3 by default
        @renamed_argument("user", "user_id", "3.0", stacklevel=4)
        def some_function(user_id, chat_id=None):
            print(f"user_id={user_id}, chat_id={chat_id}")
        some_function(user=123)  #  prints 'user_id=123, chat_id=None' with warning
        some_function(123)  #  prints 'user_id=123, chat_id=None' without warning
        some_function(user_id=123)  #  prints 'user_id=123, chat_id=None' without warning
    :param old_name:
    :param new_name:
    :param until_version: the version in which the argument is scheduled to be removed
    :param stacklevel: leave it to default if it's the first decorator used.
    Increment with any new decorator used.
    :return: decorator
    """

    def decorator(func):
        if asyncio.iscoroutinefunction(func):

            @functools.wraps(func)
            async def wrapped(*args, **kwargs):
                if old_name in kwargs:
                    warn_deprecated(
                        f"In coroutine '{func.__name__}' argument '{old_name}' "
                        f"is renamed to '{new_name}' "
                        f"and will be removed in vk.py {until_version}",
                        stacklevel=stacklevel,
                    )
                    kwargs.update({new_name: kwargs[old_name]})
                    kwargs.pop(old_name)
                return await func(*args, **kwargs)

        else:

            @functools.wraps(func)
            def wrapped(*args, **kwargs):
                if old_name in kwargs:
                    warn_deprecated(
                        f"In function `{func.__name__}` argument `{old_name}` "
                        f"is renamed to `{new_name}` "
                        f"and will be removed in vk.py {until_version}",
                        stacklevel=stacklevel,
                    )
                    kwargs.update({new_name: kwargs[old_name]})
                    kwargs.pop(old_name)
                return func(*args, **kwargs)

        return wrapped

    return decorator


def deprecated_argument(name: str, until_version: str, stacklevel: int = 3):
    def decorator(func):
        if asyncio.iscoroutinefunction(func):

            @functools.wraps(func)
            async def wrapped(*args, **kwargs):
                if name in kwargs:
                    warn_deprecated(
                        f"In coroutine '{func.__name__}' found deprecated argument '{name}'."
                        f"It's deprecated since vk.py {until_version}",
                        stacklevel=stacklevel,
                    )
                    kwargs.pop(name)
                return await func(*args, **kwargs)

        else:

            @functools.wraps(func)
            def wrapped(*args, **kwargs):
                if name in kwargs:
                    warn_deprecated(
                        f"In function '{func.__name__}' found deprecated argument '{name}'."
                        f"It's deprecated since vk.py {until_version}",
                        stacklevel=stacklevel,
                    )
                    kwargs.pop(name)
                return func(*args, **kwargs)

        return wrapped

    return decorator



from .others import SimpleResponse
from ..base import BaseModel

from ..attachments import Document

import typing


class Add(SimpleResponse):
    pass


class Delete(SimpleResponse):
    pass


class Edit(SimpleResponse):
    pass


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[Document] = None


class Get(BaseModel):
    response: GetResponse = None


class GetById(BaseModel):
    response: typing.List[Document] = None


class GetMessagesUploadServerResponse(BaseModel):
    upload_url: str = None


class GetMessagesUploadServer(BaseModel):
    response: GetMessagesUploadServerResponse = None


class GetTypesItems(BaseModel):
    id: int = None
    name: str = None
    count: int = None


class GetTypesResponse(BaseModel):
    count: int = None
    items: typing.List[GetTypesItems] = None


class GetTypes(BaseModel):
    response: GetTypesResponse = None


class GetUploadServer(BaseModel):
    response: GetMessagesUploadServerResponse = None


class GetWallUploadServer(BaseModel):
    response: GetMessagesUploadServerResponse = None


class Save(BaseModel):
    response: typing.List[Document] = None


class SearchResponse(BaseModel):
    count: int = None
    items: typing.List[Document] = None


class Search(BaseModel):
    response: SearchResponse = None



from .base import BaseMethod
from vk.types.responses import gift as m


class Gifts(BaseMethod):
    async def get(
        self, user_id: int = None, count: int = None, offset: int = None
    ):
        """
        Return a list of user gifts.
        :param user_id: User ID.
        :param count: Number of gifts to return.
        :param offset: Offset needed to return a specific subset of results.


        """
        method = self.get_method_name(self.get)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Get(**r)



from ._logging import time_logging
from .mixins import ContextInstanceMixin
from .task_manager import TaskManager
from .vkscript import Execute
from .vkscript import execute
from .vkscript import VKScriptConverter



import logging
import typing

from ..dispatcher.extension import BaseExtension
from vk.constants import JSON_LIBRARY

logger = logging.getLogger(__name__)
try:
    import aiokafka  # noqa
except ImportError:
    aiokafka = None


class Kafka(BaseExtension):
    key = "kafka"

    def __init__(
        self,
        vk,
        *topics: typing.Tuple[str],
        group_id: str,
        bootstrap_servers: str = "localhost"
    ):
        if aiokafka:
            self._vk = vk
            self.consumer = aiokafka.AIOKafkaConsumer(
                *topics,
                loop=vk.loop,
                bootstrap_servers=bootstrap_servers,
                group_id=group_id
            )
        else:
            raise RuntimeWarning(
                "Please install aiokafka (pip install aiokafka) for use this extension"
            )

    async def get_events(self) -> None:
        pass

    async def run(self, dp):
        await self.consumer.start()
        try:
            # Consume messages
            async for msg in self.consumer:
                events = JSON_LIBRARY.loads(msg.value.decode())
                if isinstance(events, list):
                    dp._process_events(events)
                else:
                    dp._process_events([events])
        finally:
            # Leave consumer group; perform autocommit if it is enabled.
            await self.consumer.stop()



import asyncio
import concurrent.futures
import functools
import multiprocessing
import typing

_pool: concurrent.futures.ThreadPoolExecutor = concurrent.futures.ThreadPoolExecutor(
    multiprocessing.cpu_count() * 5
)


class BackgroundTask:
    def __init__(
        self,
        async_or_sync: typing.Union[typing.Awaitable, typing.Callable],
        *async_or_sync_args,
        **async_or_sync_kwargs,
    ):
        """
        Run task in background.
        If task is synchronous it will be started in ThreadPoolExecutor.

        Pretty works with blocking tasks, bad works with cpu-bound tasks.

        :param async_or_sync:
        :param async_or_sync_args:
        """
        self._async_or_sync = async_or_sync
        self._async_or_sync_args = async_or_sync_args
        self._async_or_sync_kwargs = async_or_sync_kwargs
        self.is_async = asyncio.iscoroutinefunction(async_or_sync)

    async def __call__(self):
        await self.__run()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    async def __run(self) -> None:
        loop = asyncio.get_event_loop()
        if self.is_async:
            loop.create_task(
                self._async_or_sync(
                    *self._async_or_sync_args, **self._async_or_sync_kwargs
                )
            )
            return
        else:
            func = functools.partial(
                self._async_or_sync,
                *self._async_or_sync_args,
                **self._async_or_sync_kwargs,
            )
            loop.run_in_executor(_pool, func=func)



import functools

from .cached_object import CachedResponse
from vk.bot_framework import Dispatcher
from vk.bot_framework.dispatcher.storage import AbstractAsyncExpiredStorage
from vk.constants import JSON_LIBRARY
from vk.types.message import Message

"""
Now supporting only AbstractAsyncExpiredStorage-like storages.
"""


def cached_handler(
    storage: AbstractAsyncExpiredStorage, expire=10, for_specify_user=False
):
    """
    Standard caching time: 10 seconds
    :param for_specify_user: used to specify users
    :param storage: storage of cache
    :param expire: time in seconds to cache
    :return:
    """

    def wrapper(func):
        @functools.wraps(func)
        async def wrapped(*args, **kwargs):
            message: Message = args[0]
            if not isinstance(message, Message):
                raise RuntimeError(
                    "Now caching is supported only for message handlers."
                )
            if for_specify_user:
                cache_name = f"__coro_tocache:{func.__name__}:user:{message.from_id}_{message.peer_id}__"
            else:
                cache_name = f"__coro_tocache:{func.__name__}__"
            in_cache = await storage.exists(cache_name)
            if in_cache:
                cache = await storage.get(cache_name)
                cache = JSON_LIBRARY.loads(cache)
                params: dict = cache["method_params"]
                params.update(
                    {"peer_id": message.peer_id, "from_id": message.from_id}
                )
                return await Dispatcher.get_current().vk.api_request(
                    cache["method_name"], params
                )
            else:
                result = await func(*args, **kwargs)
                if not isinstance(result, CachedResponse):
                    raise ValueError(
                        "Unexpected Response. Please return 'CachedResponse' to use this decorator"
                    )
                try:
                    await storage.place(
                        cache_name,
                        JSON_LIBRARY.dumps(result.dict()),
                        expire=expire,
                    )
                except RuntimeError:
                    pass
                return result

        return wrapped

    return wrapper



import typing

from .attachments import Photo
from .base import BaseModel
from .chat import ChatPushSettings


class ConversationCanWrite(BaseModel):
    allowed: bool = None
    reason: int = None


class Peer(BaseModel):
    id: int = None
    type: str = None
    local_id: int = None


class ConversationChatSettingsACL(BaseModel):
    can_change_info: bool = None
    can_change_invite_link: bool = None
    can_change_pin: bool = None
    can_invite: bool = None
    can_promote_users: bool = None
    can_see_invite_link: bool = None
    can_moderate: bool = None


class ConversationChatSettings(BaseModel):
    members_count: int = None
    title: str = None
    pinned_message: typing.Any = None
    state: str = None
    photo: Photo = None
    active_ids: typing.List[int] = None
    admin_ids: typing.List[int] = None
    owner_id: int = None
    acl: ConversationChatSettingsACL = None


class Conversation(BaseModel):
    peer: Peer = None
    in_read: int = None
    out_read: int = None
    unread_count: int = None
    important: bool = None
    unanswered: bool = None
    push_settings: ChatPushSettings = None
    can_write: ConversationCanWrite = None
    chat_settings: ConversationChatSettings = None



from .base import BaseModel
from .attachments import Attachments

import typing


# https://vk.com/dev/objects/comment


class WallCommentThread(BaseModel):
    count: int = None
    items: typing.List["WallComment"] = None
    can_post: bool = None
    show_reply_button: bool = None
    groups_can_post: bool = None


class WallComment(BaseModel):
    id: int = None
    from_id: int = None
    date: int = None
    text: str = None
    reply_to_user: int = None
    reply_to_comment: int = None
    attachments: Attachments = None
    parents_stack: typing.List[int] = None
    thread: WallCommentThread = None


WallCommentThread.update_forward_refs()



import time
import typing

from ..dispatcher.storage import AbstractAsyncExpiredStorage

"""
Special for caching.
"""


class ExpiringDict(dict):
    def __init__(self, standart_ttl: int = 10):
        super().__init__()
        self._standart_ttl: int = standart_ttl

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)

    def set_with_ttl(self, key, value, ttl: int):
        if ttl and isinstance(ttl, int):
            if ttl == -1:
                ttl = time.time()
            value = (value, time.time() + ttl)
        else:
            value = (value, time.time() + self._standart_ttl)
        self[key] = value

    def __contains__(self, key):
        try:
            item = self.__getitem__(key, with_ttl=True)
            if item[1] < time.time():
                del self[key]
                return False
            else:
                return item
        except KeyError:
            return False

    def __getitem__(self, key, *, with_ttl=False):
        item = dict.__getitem__(self, key)
        if item[1] < time.time():
            del self[key]
            raise KeyError
        else:
            if with_ttl:
                return item
            return item[0]


class TTLDictStorage(AbstractAsyncExpiredStorage):
    def __init__(self):
        self._storage: ExpiringDict = ExpiringDict()

    async def place(
        self, key: typing.AnyStr, value: typing.Any, expire: int = -1
    ) -> None:
        if key in self._storage:
            raise RuntimeError("Storage already have this key.")
        self._storage.set_with_ttl(key, value, expire)

    async def get(
        self, key: typing.AnyStr, default: typing.Any = None
    ) -> typing.Optional[typing.Any]:
        if key in self._storage:
            return self._storage[key]
        else:
            return default

    async def delete(self, key: typing.AnyStr) -> None:
        if key in self._storage:
            del self._storage[key]
        else:
            raise RuntimeError("Undefined key.")

    async def update(
        self, key: typing.AnyStr, value: typing.Any, expire: int = -1
    ) -> None:
        if key not in self._storage:
            raise RuntimeError("Storage don`t have this key.")
        self._storage.set_with_ttl(key, value, expire)

    async def exists(self, key: typing.AnyStr):
        return key in self._storage



from typing import Callable
from typing import List
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from vk import VK


class BaseMethod:
    def __init__(self, vk: "VK", category: str):
        self.vk = vk
        self.category = category

    def __str__(self):
        return self.__dict__

    def __repr__(self):
        return "{}({!r})".format(self.__class__, self.__dict__)

    async def api_request(self, method_name: str, params: dict = None):
        return await self.vk._api_request(
            method_name, params, _raw_mode=True
        )  # noqa

    @staticmethod
    def create_params(params: dict):
        """

        :param params:
        :return:
        """
        del params["self"]
        return params

    @staticmethod
    def list_to_str(some_list: List) -> str:
        new_list = str(some_list).strip("[]")
        return new_list

    def get_method_name(self, func: Callable) -> str:
        name: str = func.__name__

        method_name = ""
        for index, elem in enumerate(name.split("_"), start=1):
            if index == 1:
                method_name += elem
                continue
            method_name += elem.title()

        return self.category + "." + method_name



from .graffiti import Graffiti
from .audio_msg import AudioMsg
from .photo import Photo, PostedPhoto, CropPhoto
from .audio import Audio
from .video import Video
from .document import Document
from .like import Like
from .market import Market, MarketPriceCurrency
from .repost import Repost
from .geo import Geo

from .attachments import Attachment, Attachments



import abc
import typing


class AbstractJsonLibrary(abc.ABC):
    def loads(self, *args, **kwargs) -> dict:
        ...

    def dumps(self, *args, **kwargs) -> typing.Union[str, bytes]:
        ...


class JsonLibrary(AbstractJsonLibrary):  # noqa
    def __init__(self, lib: AbstractJsonLibrary):
        self._library = lib

    @property
    def library(self) -> AbstractJsonLibrary:
        return self._library

    def loads(self, *args, **kwargs) -> dict:
        return self.library.loads(*args, **kwargs)

    def dumps(self, *args, **kwargs) -> typing.Union[str, bytes]:
        res = self.library.dumps(*args, **kwargs)
        if isinstance(res, bytes):
            return res.decode()
        else:
            return res
